def for_each_string_array_2_string(array=[]):
    """
    遍历字符串并加上单引号
    """
    if array:
        str = None
        for tmp in array:
            if str:
                str = str + "'" + tmp + "',"
            else:
                str = "'" + tmp + "',"
        return str[0:len(str)]


def order_attribute_operation(order_attribute):
    """
    拼接order by
    """
    if order_attribute:
        order = ",".join(order_attribute)
        return "order by " + order


class ConditionOperation(object):
    singular = ["=", ">", "<", "like"]
    allCondition = ["in"]
    twoCondition = ["between"]
    special = ["timeType"]
    attribute_type_num = "num"
    attribute_type_time = "time"

    def conditionalOperation(self, typeConditions, schemas={}):
        """
        合并conditional条件
        """
        typeConditionDict = {}
        typeOrderDict = {}
        typeSpecialSql = {}
        if typeConditions:
            for typeCondition in typeConditions:
                val = None
                type = typeCondition["type"]
                schema = schemas[type]

                if "srcNodes" in typeCondition:
                    srcNodes = typeCondition["srcNodes"]
                    if srcNodes:
                        if "src" in schema:
                            if schema["src_data_type"] == self.attribute_type_num:
                                src_node = ",".join(srcNodes)
                            else:
                                src_node = for_each_string_array_2_string(srcNodes)
                            if val:
                                val = val + schema["src"] + " in(" + src_node + ") and "
                            else:
                                val = " where " + schema["src"] + " in(" + src_node + ") and "

                if "destNodes" in typeCondition:
                    destNodes = typeCondition["destNodes"]
                    if destNodes:
                        if "dst" in schema:
                            if schema["dst_data_type"] == self.attribute_type_num:
                                dest_node = ",".join(destNodes)
                            else:
                                dest_node = for_each_string_array_2_string(destNodes)
                            if val:
                                val = val + schema["dst"] + " in(" + dest_node + ") and "
                            else:
                                val = " where " + schema["dst"] + " in(" + dest_node + ") and "
                if "id" in typeCondition:
                    srcNodes = typeCondition["id"]
                    if srcNodes:
                        if "id" in schema:
                            if schema["id_data_type"] == self.attribute_type_num:
                                ids = ",".join(srcNodes)
                            else:
                                ids = for_each_string_array_2_string(srcNodes)
                            if val:
                                val = val + schema["id"] + " in(" + ids + ") and "
                            else:
                                val = " where " + schema["id"] + " in(" + ids + ") and "
                if "conditions" in typeCondition:
                    conditions = typeCondition["conditions"]
                    val_sql = val
                    val_dict = self.splice_condition(conditions, schema, val_sql)
                    if "sql" in val_dict:
                        typeSpecialSql["sql"] = val_dict["sql"]
                    val = val_dict["val"]

                if val:
                    typeConditionDict[type] = val + " 1=1"

                if "orderAttribute" in typeCondition:
                    type_order = typeCondition["orderAttribute"]
                    order_val = order_attribute_operation(type_order)
                    typeOrderDict[type] = order_val
        return typeConditionDict, typeOrderDict, typeSpecialSql

    def splice_condition(self, conditions, schema, val_sql=None):
        """
        拼接条件
        """
        if conditions:
            tem_dict = {}
            val = val_sql if val_sql else " where "
            for condition in conditions:
                symbol = condition["symbol"]
                attribute_type = condition["type"]
                if symbol in self.singular:
                    if attribute_type == self.attribute_type_num:
                        val = val + condition["attribute"] + condition["symbol"] \
                              + condition["conditional"][0] + " and "
                    elif attribute_type == self.attribute_type_time:
                        val = val + " formatDateTime(" + condition["attribute"] + ",'%F %T') " + condition["symbol"] \
                              + " '" + condition["conditional"][0] + "' and "
                    else:
                        val = val + condition["attribute"] + condition["symbol"] \
                              + "'" + condition["conditional"][0] + "' and "

                if symbol in self.allCondition:
                    if attribute_type == self.attribute_type_num:
                        val = val + condition["attribute"] + condition["symbol"] + "(" \
                              + ",".join(condition["conditional"]) + ")" + " and "
                    else:
                        val = val + condition["attribute"] + condition["symbol"] + "(" \
                              + for_each_string_array_2_string(condition["conditional"]) + ")" + " and "
                if symbol in self.twoCondition:
                    if attribute_type == self.attribute_type_num:
                        val = val + condition["attribute"] + " between " \
                              + condition["conditional"][0] + " and " \
                              + condition["conditional"][1] + " and "
                    elif attribute_type == self.attribute_type_time:
                        val = val + " formatDateTime(" + condition["attribute"] + ",'%F %T') " + condition["symbol"] \
                              + " '" + condition["conditional"][0] + "' and "
                    else:
                        val = val + condition["attribute"] + " between '" \
                              + condition["conditional"][0] + "' and '" \
                              + condition["conditional"][1] + "' and "
                if symbol in self.special:
                    sql = self.time_line_count_sql(condition, schema, val_sql)
                    tem_dict["sql"] = sql
            tem_dict["val"] = val
            return tem_dict

    def time_line_count_sql(self, condition, schema, val_sql):
        """
        处理特殊sql的拼接
        """
        if "id" in schema and "label" in schema:
            attribute = schema["id"] + " , " + schema["label"]
        else:
            attribute = schema["src"] + " , " + schema["dst"] + " , " + schema["rank"]
        if val_sql:
            val_sql = val_sql + " 1=1 "
        else:
            val_sql = " where 1=1 "
        if condition["conditional"][0] == "year":
            sql = "select toYear(" + condition["attribute"] + ") year,count(1) count from ( select distinct " \
                  + attribute + "," + condition["attribute"] + " from " + schema["db"] + "." + schema[
                      "table"] + val_sql + " ) group by year order by year "
        elif condition["conditional"][0] == "quarter":
            sql = "select concat(toString(year),\' \',toString(quarter)) yearQuarter ,count from (select toYear(" + \
                  condition["attribute"] + ") year,toQuarter(" + condition["attribute"] \
                  + ") quarter ,count(1) count from   ( select distinct  " + attribute + "," + condition[
                      "attribute"] + " from " \
                  + schema["db"] + "." + schema["table"] \
                  + val_sql \
                  + " )  group by year,quarter order by year,quarter) "
        elif condition["conditional"][0] == "month":
            sql = "select concat(toString(year),\' \',toString(month)) yearMonth, count from ( select toYear(" + \
                  condition["attribute"] + ") year,toMonth(" + condition["attribute"] \
                  + ") month ,count(1) count from  ( select distinct " + attribute + "," + condition[
                      "attribute"] + " from " \
                  + schema["db"] + "." + schema["table"] \
                  + val_sql \
                  + " )group by year,month order by year,month)"
        elif condition["conditional"][0] == "week":
            sql = "select concat(toString(yw),\' \',toString(week)) ywWeek, count from (select toYearWeek(" + \
                  condition[
                      "attribute"] + ") yw, toDayOfWeek(" + condition["attribute"] + \
                  ") week ,count(1) count from   ( select distinct  " + attribute + "," + condition[
                      "attribute"] + " from " \
                  + schema["db"] + "." + schema["table"] \
                  + val_sql \
                  + ") group by yw,week order by yw,week )"
        elif condition["conditional"][0] == "day":
            sql = "select formatDateTime(" + condition[
                "attribute"] + ",'%F') day ,count(1) count from  ( select distinct  " \
                  + attribute + "," + condition["attribute"] + " from " \
                  + schema["db"] + "." + schema["table"] + val_sql + " ) group by day order by day"
        elif condition["conditional"][0] == "hours":
            sql = "select formatDateTime(" + condition[
                "attribute"] + ",'%F %H') h ,count(1) count from ( select distinct  " \
                  + attribute + "," + condition["attribute"] + " from " \
                  + schema["db"] + "." + schema["table"] + val_sql + " )group by h order by h"
        elif condition["conditional"][0] == "minute":
            sql = "select formatDateTime(" + condition[
                "attribute"] + ",'%F %R') m ,count(1) count from  ( select distinct  " \
                  + attribute + "," + condition["attribute"] + " from " \
                  + schema["db"] + "." + schema["table"] + val_sql + " )group by m order by m"
        else:
            sql = "select formatDateTime(" + condition[
                "attribute"] + ",'%F') day ,count(1) count from  ( select distinct  " \
                  + attribute + "," + condition["attribute"] + " from " \
                  + schema["db"] + "." + schema["table"] + val_sql + " )group by day order by day"

        return sql

    def conditional_operation_by_one(self, condition, schemas={}):
        """
        合并conditional条件
        """
        typeConditionDict = {}
        typeOrderDict = {}
        typeSpecialSql = {}

        val = None
        node_type = condition.get("type")
        schema = schemas.get(node_type)
        src_nodes = condition.get("srcNodes")
        if src_nodes:
            if "src" in schema:
                if schema["src_data_type"] == self.attribute_type_num:
                    src_node = ",".join(src_nodes)
                else:
                    src_node = for_each_string_array_2_string(src_nodes)
                if val:
                    val = val + schema["src"] + " in(" + src_node + ") and "
                else:
                    val = " where " + schema["src"] + " in(" + src_node + ") and "

        dest_nodes = condition.get("destNodes")
        if dest_nodes:
            if "dst" in schema:
                if schema["dst_data_type"] == self.attribute_type_num:
                    dest_node = ",".join(dest_nodes)
                else:
                    dest_node = for_each_string_array_2_string(dest_nodes)
                if val:
                    val = val + schema["dst"] + " in(" + dest_node + ") and "
                else:
                    val = " where " + schema["dst"] + " in(" + dest_node + ") and "

        _id = condition.get("id")
        if _id:
            if "id" in schema:
                if schema["id_data_type"] == self.attribute_type_num:
                    ids = ",".join(_id)
                else:
                    ids = for_each_string_array_2_string(_id)
                if val:
                    val = val + schema["id"] + " in(" + ids + ") and "
                else:
                    val = " where " + schema["id"] + " in(" + ids + ") and "

        sub_conditions = condition.get("conditions")
        val_sql = val
        val_dict = self.splice_condition(sub_conditions, schema, val_sql)
        if "sql" in val_dict:
            typeSpecialSql["sql"] = val_dict["sql"]

        val = val_dict["val"]

        if val:
            typeConditionDict[node_type] = val + " 1=1"

        type_order = condition.get("orderAttribute")
        order_val = order_attribute_operation(type_order)
        typeOrderDict[node_type] = order_val
        return typeConditionDict, typeOrderDict, typeSpecialSql



import requests
import json

if __name__ == '__main__':

# graph deletion
    url_graph_deletion = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/graph-deletion"
    data_graph_deletion = json.dumps({
        "graph_name": "twitter_v1"
    })
    response = requests.delete(url=url_graph_deletion, data=data_graph_deletion)
    print(response.text)


# match edge
    url_edges = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/edges"
    data_edges = json.dumps({
        "graph_name": "twitter_v1",
        "edge_name": "follow",
        "edge_con_list": ["follow_date>'2010-12-31'"],
        "target_field_list": ["follow_date"],
        "data_type": "df"
    })
    response = requests.get(url=url_edges, data=data_edges)
    print(response.text)


# multi-hop
    url_multi_hop = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/multi-hop"

    data_multi_hop = json.dumps({
        "graph_name": "twitter_v1",
        "step": 1,
        "start_vertex_list": ['10000082','10000162'],
        "edge_name_list": ["follow"],
        "direction": "forward",
        "edge_con_list_list": [["follow_date>'2010-06-30'"]],
        "target_field_list": ["follow_date"],
        "only_last_step": False,
        "plus_last_vertexes": False
    })
    response = requests.get(url=url_multi_hop, data=data_multi_hop)
    print(response.text)


# one-hop
    url_one_hop = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/one-hop"
    data_one_hop = json.dumps({
        "graph_name": "twitter_v1",
        "step": 1,
        "start_vertex_list": ['10000082','10000162'],
        "edge_name_list": ["follow"],
        "direction": "forward",
        "edge_con_list_list": [["follow_date>'2010-06-30'"]],
        "target_field_list": ["follow_date"],
        "only_last_step": False,
        "plus_last_vertexes": False
    })
    response = requests.get(url=url_one_hop, data=data_one_hop)
    print(response.text)


# match multi_hop_common_vertices
    url_multi_hop_common_vertices = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/multi-hop-common-vertices"

    data_multi_hop_common_vertices = json.dumps({
        "step": 1,
        "start_vertex_list": [
            '10000082',
            '10000162'
        ],
        "edge_name_list": [
            "follow"
        ],
        "graph_name": "twitter_v1",
        "direction": "forward",
        "edge_con_list_list": [
            ["follow_date>'2010-06-30'"]
        ]
    })
    response = requests.get(url=url_multi_hop_common_vertices, data=data_multi_hop_common_vertices)
    print(response.text)


# path
    url_path = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/path"
    data_path = json.dumps({
        "graph_name": "twitter_v1",
        "start_vertex_list": [
            '10000082',
            '10000162'
        ],
        "end_vertex_list": ['49265462', '9170712', '42506240', '28088190'],
        "edge_name_list": ["follow"],
        "edge_con_list_list": [["follow_date>'2010-06-30'"]],
        "target_field_list": ["follow_date"],
        "step_limit": 1
    })
    response = requests.get(url=url_path, data=data_path)
    print(response.text)


# graph registration
    url_graph_registration = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/graph-registration"
    data_graph_registration = json.dumps({
        "graph_name": "twitter_v1",
        "graph_cfg": "{\"edges\":{\"follow\":{\"db\":\"twitter_v1\",\"table\":\"follow\",\"src\":\"followee_id\",\"dst\":\"follower_id\",\"fields\":[\"follow_date\"]}},\"vertexes\":{}}"
    })
    response = requests.post(url=url_graph_registration, data=data_graph_registration)
    print(response.text)


# graph show
    url_graph_show = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/graph-show"
    data_graph_show = json.dumps({})
    response = requests.get(url=url_graph_show, data=data_graph_show)
    print(response.text)


# graph summary
    url_graph_summary = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/graph-summary/twitter_v1"
    # data_graph_summary = json.dumps({
    #     "graph_name": "livejournal_v1"
    # })
    response = requests.get(url=url_graph_summary)
    print(response.text)


# graph description
    url_graph_description = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/graph-description/twitter_v1"
    # data_graph_description = json.dumps({
    #     "graph_name": "livejournal_v1"
    # })
    response = requests.get(url=url_graph_description)
    print(response.text)


# subgraph creation
    url_subgraph_creation = "p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/subgraph-creation"
    data_subgraph_creation = json.dumps({
        "graph_name": "twitter_v1",
        "subgraph_name": "twitter_v1_sub"
    })
    response = requests.get(url=data_subgraph_creation)
    print(response.text)


# subgraph destruction
    url_subgraph_destruction = "p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/subgraph-destruction"
    data_subgraph_destruction = json.dumps({
        "graph_name": "twitter_v1",
        "subgraph_name":"twitter_v1_sub"
    })
    response = requests.get(url=data_subgraph_destruction)
    print(response.text)

# subgraph update multi hop
    url_subgraph_update_multi_hop = "p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/subgraph-update-multi-hop"
    data_subgraph_update_multi_hop = json.dumps({
            "graph_name": "twitter_v1",
            "subgraph_name":"twitter_v1_sub",
            "step": 1,
            "start_vertex_list": ['10000082','10000162'],
            "edge_name_list": ["follow"],
            "direction": "forward",
            "edge_con_list_list": [["follow_date>'2010-06-30'"]]
    })
    response = requests.get(url=data_subgraph_update_multi_hop)
    print(response.text)

# subgraph update path
    url_subgraph_update_path = "p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/subgraph-update-path"
    data_subgraph_update_path = json.dumps({
            "graph_name": "twitter_v1",
            "subgraph_name":"twitter_v1_sub",
            "start_vertex_list": [
                '10000082',
                '10000162'
            ],
            "end_vertex_list": ['49265462', '9170712', '42506240', '28088190'],
            "edge_name_list": ["follow"],
            "edge_con_list_list": [["follow_date>'2010-06-30'"]],
            "step_limit": 1
    })
    response = requests.get(url=data_subgraph_update_path)
    print(response.text)


# subgraph update edge
    url_subgraph_update_edge = "p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/subgraph-update-edge"
    data_subgraph_update_edge = json.dumps({
            "graph_name": "twitter_v1",
            "subgraph_name":"twitter_v1_sub",
            "edge_name": "follow",
            "edge_con_list": ["follow_date>'2010-12-31'"],
            "target_field_list": ["follow_date"],
            "data_type": "df"
    })
    response = requests.get(url=data_subgraph_update_edge)
    print(response.text)



from daos.graph_add_op import CHGraph
from utils.ck_client import get_client
from utils.decorators import log_execute_time
from daos.graph_op import CHGraph as ch
from config.logger_design import get_logger
from config.config import ClusterConfig
from services.common import get_graph_vertexes, get_graph_edges
from services.sql_condition_operation import ConditionOperation
from utils.sql_replace_operation import SqlReplaceHandler
from celery_tasks.tasks import graph_calculation_task

cluster_config = ClusterConfig()
sql_replace_handler = SqlReplaceHandler(cluster_config.is_cluster)
logger = get_logger()

operation_dict = {'avg': 'avg', 'sum': 'sum', 'min': 'min', 'max': 'max', 'count': 'count', }


def disconnect_client(client):
    if client:
        client.disconnect()


class ModelService(object):
    calculations = ["Pagerank", "Closeness", "Betweenness"]
    # 切换图空间
    def use_graph(self, graph_name, db):
        res = db.use_tables(graph_name)
        print(res)
        if res is None:
            logger.warning("graph name [" + graph_name + "] does not exist")
            return
        else:
            self.graph_name = graph_name
            self.graph_cfg = res
            logger.info("use graph [" + graph_name + "] done")

    def get_graph_task(self, graph_name, db):
        res = db.get_task(graph_name)
        print(res)
        if res is None:
            logger.warning("get_graph_task ,graph name [" + graph_name + "] does not exist")
            return
        else:
            logger.info("get_graph_task  [" + graph_name + "] done")
            return ",".join(('%s' %num for num in res))


    def search_subgraph_by_condition(self, data, config_param):
        """
        根据条件过滤查询子图
        """
        graph_name = data.get("subGraph")
        if not graph_name:
            return

        edge_types = data.get("edgeTypes")
        node_types = data.get("nodeTypes")

        db = config_param["db"]
        clickhouse_connect = config_param["clickhouse_connect"]
        graph_client = get_client(clickhouse_connect)
        graph = CHGraph(graph_client)
        self.use_graph(graph_name, db)

        edges = self.graph_cfg["edges"]
        vertexes = self.graph_cfg["vertexes"]

        if "edgeConditions" in data.keys():
            edge_conditions = data["edgeConditions"]
            edge_condition_dict, edge_order_dict, special_sql_dict = ConditionOperation().conditionalOperation(
                edge_conditions, edges)
        else:
            edge_condition_dict, edge_order_dict = {}, {}
        if "nodeConditions" in data.keys():
            node_conditions = data["nodeConditions"]
            node_condition_dict, node_order_dict, special_sql_dict = ConditionOperation().conditionalOperation(
                node_conditions, vertexes)
        else:
            node_condition_dict, node_order_dict = {}, {}

        ext_field_list = data.get("fieldList")
        result_type = data.get("resultType")

        data = {}
        path_data = {}
        edge_data_list = []
        vertexes_data_list = []

        if edge_types:
            edge_data_list = execute_controller(edge_types, edges, edge_condition_dict, edge_order_dict, graph, "edge",
                                               ext_field_list, result_type)

        if node_types:
            vertexes_data_list = execute_controller(node_types, vertexes, node_condition_dict, node_order_dict, graph,
                                                   "vertexes", ext_field_list, result_type)

        path_data["graphEdges"] = edge_data_list
        path_data["graphNodes"] = vertexes_data_list
        data["pathList"] = path_data
        # 释放链接
        disconnect_client(graph_client)
        return data

    def time_line_search(self, data, config_param):
        """
        时间轴获取点边的模型
        """
        res_data = {}
        graph_name = data.get("subGraph")
        if not graph_name:
            raise ValueError("subGraph param is None")

        db = config_param["db"]
        clickhouse_connect = config_param["clickhouse_connect"]
        graph_client = get_client(clickhouse_connect)
        graph = CHGraph(graph_client)
        self.use_graph(graph_name, db)

        rest = []
        edges = self.graph_cfg.get("edges")
        vertexes = self.graph_cfg.get("vertexes")
        condition_operation = ConditionOperation()
        if "edgeConditions" in data:
            edge_conditions = data.get("edgeConditions") or []
            for condition in edge_conditions:
                edge_condition_dict, edge_order_dict, special_sql_dict = condition_operation.\
                    conditional_operation_by_one(condition, edges)
                if "sql" in special_sql_dict:
                    res = execute_graph_sql(graph, special_sql_dict.get("sql"), type=condition.get("type"))
                    rest.append(res)
        elif "nodeConditions" in data:
            node_conditions = data.get("nodeConditions") or []
            for condition in node_conditions:
                node_condition_dict, node_order_dict, special_sql_dict = condition_operation.\
                    conditional_operation_by_one(condition, vertexes)
                if "sql" in special_sql_dict:
                    res = execute_graph_sql(graph, special_sql_dict.get("sql"), type=condition.get("type"))
                    rest.append(res)

        if rest:
            row_data = []
            for item in rest:
                result = item.get("data")
                node_type = item.get("type")
                schema = result.get("schema") or []
                detail = result.get("detail") or []
                schema.insert(0, "type")
                if "columns" not in res_data:
                    res_data["columns"] = schema

                for ele in detail:
                    ele.insert(0, node_type)
                row_data.extend(detail)

            res_data["rowList"] = row_data

        # 释放链接
        disconnect_client(graph_client)
        return res_data

    def count_src_dst_round(self, data, config_param):
        """
        统计任意两点间的边
        """
        graph_name = data.get("subGraph")
        edge_types = data.get("edgeTypes")
        if not graph_name:
            return

        db = config_param["db"]
        clickhouse_connect = config_param["clickhouse_connect"]
        graph_client = get_client(clickhouse_connect)
        graph = CHGraph(graph_client)
        self.use_graph(graph_name, db)

        edges = self.graph_cfg["edges"]
        if "edgeConditions" in data:
            edge_conditions = data.get("edgeConditions")
            edge_condition_dict, edge_order_dict, special_sql_dict = ConditionOperation().conditionalOperation(
                edge_conditions, edges)
        else:
            edge_condition_dict, edge_order_dict = {}, {}

        if edge_types:
            edge_data_list = execute_controller(edge_types, edges, edge_condition_dict, edge_order_dict, graph,
                                               "edgeCount")
        else:
            edge_data_list = execute_controller(edges.keys(), edges, edge_condition_dict, edge_order_dict, graph,
                                               "edgeCount")
        res_data = {}
        res_list = []
        if edge_data_list:
            for res_dict in edge_data_list:
                res = res_dict["data"]
                res_data["columns"] = res["schema"]
                res_data["rowList"] = res["detail"]
                res_data["type"] = res_dict["type"]
                res_list.append(res_data)
        # 释放链接
        disconnect_client(graph_client)
        return res_list

    @log_execute_time(logger)
    def query_subgraph(self, data, config_params):
        """
        根据条件过滤查询子图
        """
        graphName = data.get("subGraph")
        if not graphName:
            raise Exception("subGraph name is None")

        db = config_params["db"]
        clickhouse_connect = config_params["clickhouse_connect"]
        graph_client = get_client(clickhouse_connect)
        graph = ch(graph_client)
        graph.use_graph(graphName, db)
        graph_cfg = graph.graph_cfg

        vertexes = get_graph_vertexes(graph_cfg, graph)
        edges = get_graph_edges(graph_cfg, graph)

        res_list = {}
        res_list["pathList"] = {}
        res_list["pathList"]["graphEdges"] = edges
        res_list["pathList"]["graphNodes"] = vertexes
        return res_list

    def statistics_operation_function(self, data, config_param):
        """
        统计子图中点或者边的数量
        """
        data = dispose_parse_params(data)
        graph_name = data.get("subGraph")
        graph_type = data.get("type")
        orderTypes = data.get("orderTypes") or ""
        if not graph_name:
            return

        db = config_param["db"]
        clickhouse_connect = config_param["clickhouse_connect"]
        graph_client = get_client(clickhouse_connect)
        graph = CHGraph(graph_client)

        operations = {"1": "count"}
        group_field = []
        order_operation = orderTypes
        field_list = []
        self.use_graph(graph_name, db)
        if graph_type:
            schemas = self.graph_cfg[graph_type]
            res = None
            for schema in schemas:
                execute_sql = join_sql(operations, group_field, order_operation,
                                      schema, schemas, field_list, graph_type)
                result_dict = execute_graph_sql(graph, execute_sql)
                res = self.dispose_statistics_operation_res(result_dict, res, schema)
            result = {graph_type: res}
        else:
            edges = self.graph_cfg["edges"]
            vertexes = self.graph_cfg["vertexes"]
            res_edge = None
            res_vertex = None
            for edge in edges:
                execute_sql = join_sql(operations, group_field, order_operation,
                                      edge, edges, field_list, "edges")
                result_dict = execute_graph_sql(graph, execute_sql)
                res_edge = self.dispose_statistics_operation_res(result_dict, res_edge, edge)
            for vertex in vertexes:
                execute_sql = join_sql(operations, group_field, order_operation,
                                      vertex, vertexes, field_list, "vertexes")
                result_dict = execute_graph_sql(graph, execute_sql)
                res_vertex = self.dispose_statistics_operation_res(result_dict, res_vertex, vertex)
            result = {"vertexes": res_vertex, "edges": res_edge}
        return result

    @staticmethod
    def dispose_statistics_operation_res(result_dict, res, type):
        if "typeValue" not in result_dict["data"]["schema"]:
            result_dict["data"]["schema"].append("typeValue")
        result_dict["data"]["detail"][0].append(type)
        if res:
            res["data"]["detail"].append(result_dict["data"]["detail"][0])
        else:
            res = result_dict
        return res

    def statistics_operation_attributes_function(self, data, config_param):
        """
        统计子图中点或者边的多个属性
        """
        data = dispose_parse_params(data)
        graph_name = data.get("subGraph")
        type_value = data.get("type")
        attributes = data.get("attributes")
        orderTypes = data.get("orderTypes") or ""
        if not graph_name or not attributes:
            return None

        # 获取客户端
        db = config_param["db"]
        clickhouse_connect = config_param["clickhouse_connect"]
        graph_client = get_client(clickhouse_connect)
        graph = CHGraph(graph_client)

        # 更新子图
        self.use_graph(graph_name, db)
        edges = self.graph_cfg["edges"]
        vertexes = self.graph_cfg["vertexes"]

        # 处理参数
        attribute_list = attributes.split(",")
        order_list = orderTypes.split(",") if orderTypes else ""

        res_list = []
        for i in range(len(attribute_list)):
            order_operation = order_list[i] if order_list else ""
            data_param = {"subGraph": graph_name, "type": type_value, "attribute": attribute_list[i],
                          "orderTypes": order_operation}
            if type_value in edges:
                index = edges[type_value]["fields"].index(attribute_list[i])
                field_type = edges[type_value]["types"][index]
                if field_type == "String":
                    data_param["statistics"] = "count"
                elif field_type == "Date":
                    data_param["statistics"] = "count"

            if type_value in vertexes:
                index = vertexes[type_value]["fields"].index(attribute_list[i])
                field_type = vertexes[type_value]["types"][index]
                if field_type == "String":
                    data_param["statistics"] = "count"
                elif field_type == "Date":
                    data_param["statistics"] = "count"
            res_one = self.statistics_operation_attribute_function(data_param, config_param, graph=graph)
            res_list.append(res_one)

        return res_list

    def statistics_operation_attribute_function(self, data, config_param, label_statistic=None, graph=None):
        """
        统计点或者边的单个属性
        """
        data = dispose_parse_params(data)
        graph_name = data.get("subGraph")
        type_value = data.get("type")
        attribute = data.get("attribute")
        statistics = data.get("statistics") or "def"
        orderTypes = data.get("orderTypes") or ""
        if not graph_name or not attribute:
            return

        # 处理参数
        attribute_arr = attribute.split(",")
        statistics_arr = statistics.split(",")
        if len(statistics_arr) != len(attribute_arr):
            return

        operations = {}
        for i in range(len(statistics_arr)):
            operations[attribute_arr[i]] = statistics_arr[i]
        if "def" in statistics_arr:
            group_field = None
        else:
            group_field = attribute_arr
        order_operation = orderTypes
        field_list = []

        # 获取客户端
        db = config_param["db"]
        clickhouse_connect = config_param["clickhouse_connect"]
        graph_client = get_client(clickhouse_connect)
        if not graph:
            graph = CHGraph(graph_client)

        # 更换子图
        self.use_graph(graph_name, db)
        edges = self.graph_cfg["edges"]
        vertexes = self.graph_cfg["vertexes"]
        if type_value in edges:
            edge_schema = edges[type_value]
            if label_statistic:
                group_field = [edge_schema["src"], edge_schema["dst"]]
            execute_sql = join_sql(operations, group_field, order_operation,
                                  type_value, edges, field_list, "edges")
        elif type_value in vertexes:
            vertex_schema = vertexes[type_value]
            if label_statistic:
                group_field = [vertex_schema["id"]]
            execute_sql = join_sql(operations, group_field, order_operation,
                                  type_value, vertexes, field_list, "vertexes")
        else:
            return None
        result_dict = execute_graph_sql(graph, execute_sql)

        return result_dict


    def calculation_attribute_select_function(self, data, config_param, label_statistic=None, graph=None):
        """
        计算属性查询
        """
        data = dispose_parse_params(data)
        graph_name = data.get("subGraph")
        compute = data.get("compute")
        task_id = data.get("taskId")
        if not graph_name:
            return
        # 获取客户端
        db = config_param["db"]
        clickhouse_connect = config_param["clickhouse_connect"]
        graph_client = get_client(clickhouse_connect)
        if not graph:
            graph = CHGraph(graph_client)

        # 更换子图
        # self.use_graph(graph_name, db)
        computes, task_str = self.execute_params_2_calculation(compute, task_id)
        if task_str is None:
            task_str = self.get_graph_task(graph_name, db)
        if task_str is None:
            return None
        column_str = ""
        for com in computes:
            column_str = column_str + " ,sumIf(val,graph_algorithm_type = '"+com+"') as  " + com

        execute_sql = "select id ,id_type " + column_str + "  from " + graph_name +\
                      ".calculation_attribute where task_id in " \
                      " ( "+task_str + ") group by id,id_type"
        result_dict = execute_graph_sql(graph, execute_sql)

        return result_dict


    def calculation_attribute_submit_function(self, data, config_param, label_statistic=None, graph=None):
        """
        计算任务提交
        """
        data = dispose_parse_params(data)
        graph_name = data.get("subGraph")
        compute = data.get("compute")
        task_id = data.get("taskId")
        logger.info("subGraph:"+graph_name+"   compute:"+compute+"  task_id:"+str(task_id))
        if not graph_name or not task_id or not compute:
            return None
        # 获取客户端
        db = config_param["db"]
        clickhouse_connect = config_param["clickhouse_connect"]
        graph_client = get_client(clickhouse_connect)
        if not graph:
            graph = CHGraph(graph_client)
        #对接计算任务接口调用
        try:
            graph_calculation_task.delay(graph_name, task_id, compute)
        except Exception as e:
            print(e)
            logger.error(e)
            return "task filed"
        return "success"

    def execute_params_2_calculation(self, compute, task_id):
        """
        处理计算参数
        """
        if compute and task_id:
            return [compute], "\'"+str(task_id)+"\'"
        else:
            return self.calculations,None


def join_sql(operations, group_field, order_operation,
            type_vertex, schema, field_list, subgraph_type):
    """
    拼接sql,查询表数量
    """
    field_list = field_list if field_list else []
    for operation in operations:
        if operation in field_list:
            continue
        field_list.append(operation)

    if group_field:
        for field in group_field:
            if field in field_list:
                continue
            field_list.append(field)

    operList = statistics_operation(operations)
    operStr = ','.join(operList)
    group_s = ""
    operStr_new = operStr
    if group_field:
        group_str = ','.join(group_field)
        operStr_new = group_str + "," + operStr_new
        group_s = " group by " + group_str
    main_sql = main_sql_value(type_vertex, schema, field_list, subgraph_type)
    if main_sql:
        sql = "select " + operStr_new + " from (" + main_sql + ")" + group_s + " order by " + operStr + "  " + order_operation
        return sql


def statistics_operation(operations):
    """
    处理Statistics函数
    """
    operList = []
    for operation in operations:
        oper = operations[operation]
        if oper in operation_dict:
            operList.append(operation_dict[oper] + '(' + operation + ')')
        else:
            operList.append(operation)
    return operList


def main_sql_value(type_value, schema, field_list, subgraph_type):
    """
    点子图主sql
    """
    sql = None
    if type_value in schema:
        value = schema[type_value]
        if subgraph_type == "vertexes":
            if field_list:
                fields = ",".join(field_list)
                sql = "select distinct " + value["id"] + "," + fields + " from " + value["db"] + "." + value["table"]
            else:
                sql = "select distinct " + value["id"] + "," + " from " + value["db"] + "." + value["table"]
        else:
            if field_list:
                fields = ",".join(field_list)
                sql = "select distinct " + value["src"] + "," + value["dst"] + "," + value["rank"] + "," + fields \
                      + " from " + value["db"] + "." + value["table"]
            else:
                sql = "select distinct " + value["src"] + "," + value["dst"] + "," + value["rank"] + "," + " from " \
                      + value["db"] + "." + value["table"]
    return sql


def execute_controller(array, schema, condition_dict, order_dict, graph, type,
                      ext_field_list=None, result_type=None):
    """
    执行多条sql，并拼接返回
    """
    result = []
    for key in array:
        data = schema[key]
        condition = condition_splice(key, condition_dict)
        order = condition_splice(key, order_dict)
        if type == "edge":
            main_sql = edges_splice(data, ext_field_list)
        elif type == "edgeCount":
            main_sql = edges_count_splice(data)
            group_splice = edges_group_splice(data)
        else:
            main_sql = vertexes_splice(data, ext_field_list)

        if main_sql:
            if type == "edgeCount":
                sql = main_sql + condition + group_splice + order
            else:
                sql = main_sql + condition + order
            vertexes_data = execute_graph_sql(graph, sql, key, result_type)
            result.append(vertexes_data)
    return result


def edges_group_splice(edges_schema):
    src = edges_schema.get("src")
    dst = edges_schema.get("dst")

    if not src or not dst:
        return

    group_splice = " group by " + src + "," + dst + " "
    return group_splice


def execute_graph_sql(graph, sql, type=None, result_type=None):
    """
    组合sql执行，返回图对象
    """
    result_dict = {}
    new_sql = sql_replace_handler.deal_sql(sql)
    logger.info("[execute_graph_sql] is_cluster:{}      sql:{}".format(sql_replace_handler.is_cluster, new_sql))
    res = graph.execute(new_sql)

    if result_type:
        result_dict["data"] = res
    else:
        data = {}
        schemas = res.columns.values.tolist()
        field_data = res.values.tolist()
        data["schema"] = schemas
        data["detail"] = field_data
        result_dict["data"] = data

    if type:
        result_dict["type"] = type
    return result_dict


def condition_splice(type, condition={}):
    """
    拼接条件
    """
    if len(condition) > 0 and type in condition:
        return condition[type]
    return ""


def vertexes_splice(vertexes_schema, ext_field_list=None):
    """
    拼接点主体查询sql
    """
    db = vertexes_schema.get("db") or "default"
    table = vertexes_schema.get("table")
    id = vertexes_schema.get("id")
    label = vertexes_schema.get("label")

    for item in [table, id, label]:
        if not item:
            return

    if not ext_field_list:
        ext_field_list = vertexes_schema.get("fields")

    for item in [id, label]:
        if item in ext_field_list:
            ext_field_list.remove(item)

    ext_field = ",".join(ext_field_list) if ext_field_list else None
    if ext_field:
        main_sql = "select distinct " + id + "," + label + ", " + ext_field + " from " + db + "." + table
    else:
        main_sql = "select distinct " + id + "," + label + " from " + db + "." + table
    return main_sql


def edges_splice(edges_schema, ext_field_list=None):
    """
    拼接边主体查询sql
    """
    db = edges_schema.get("db") or "default"
    table = edges_schema.get("table")
    src = edges_schema.get("src")
    dst = edges_schema.get("dst")
    rank = edges_schema.get("rank")

    for item in [table, src, dst, rank]:
        if not item:
            return

    if not ext_field_list:
        ext_field_list = edges_schema.get("fields")

    for item in [src, dst, rank]:
        if item in ext_field_list:
            ext_field_list.remove(item)

    ext_field = ",".join(ext_field_list) if ext_field_list else None
    if ext_field:
        main_sql = "select distinct " + src + ", " + dst + "," + rank + " ," + ext_field + "  from  " + db + "." + table
    else:
        main_sql = "select distinct " + src + ", " + dst + "," + rank + "  from  " + db + "." + table
    return main_sql


def edges_count_splice(edges_schema):
    """
    拼接边主体查询sql
    """
    db = edges_schema.get("db") or "default"
    table = edges_schema.get("table")
    src = edges_schema.get("src")
    dst = edges_schema.get("dst")
    rank = edges_schema.get("rank")

    for item in [table, src, dst, rank]:
        if not item:
            return

    main_sql = "select " + src + ", " + dst + ", count(1) count  from  " + db + "." + table
    return main_sql


def group_attribute_operation(group_attribute):
    """
    拼接group by
    暂时不用，没有处理聚合函数
    """
    if group_attribute:
        group = ",".join(group_attribute)
        return "group by " + group


def dispose_parse_params(data):
    """
    处理parse.parse_qs方法获取的get参数
    """
    for key in data.keys():
        if isinstance(data[key], list):
            data[key] = data[key][0]
    return data


def main():
    from clickhouse_driver import Client
    graphClient = Client(host="10.202.255.93", port="9090", user="default", password="root")
    from daos.graph_op import CHGraph
    graph = CHGraph(graphClient)
    res = graph.execute("select * from anti_money_launder.transactions limit 10")
    list = res.columns.values.tolist();
    print(res.values.tolist())
    print(list)


def test():
    from config import data_load_config as config
    config_params = config.load_config()
    str(config_params["db"])
    print(str(config_params["db"]))
    dict_data = {"subGraph": "anti_money_launder"}
    model_service = ModelService()
    result_dict = model_service.search_subgraph_by_condition(dict_data, config_params)
    print(result_dict)


def test_function():
    from config import data_load_config as config
    config_params = config.load_config()
    str(config_params["db"])
    print(str(config_params["db"]))

    dict_data = {"subGraph": "0b58bfda_7bf5_4661_ae21_5079057afe49", "type": "xiqianvertex",
                 "attributes": "type,acct_stat"}
    model_service = ModelService()
    result_dict = model_service.statistics_operation_attributes_function(dict_data, config_params)
    print(result_dict)


if __name__ == '__main__':
    # main()
    test_function()



from __future__ import print_function

import sys
import os
curPath = os.path.abspath(os.path.dirname(__file__))
rootPath = os.path.split(curPath)
sys.path.append(rootPath[0])
sys.path.append(rootPath[0] + "/" + rootPath[1])

print(sys.path)

import tornado.ioloop
import tornado.web

from tornado_swagger.model import register_swagger_model

from config.logger_design import get_logger
from config.data_load_config import load_config
from services.model import model_service_graph_search_one_hop, \
    model_service_graph_search_multi_hop, \
    model_service_graph_search_multi_hop_multi_edge, \
    model_service_graph_search_one_hop_multi_edge, \
    model_service_graph_search_multi_hop_common_vertexes, \
    model_service_graph_search_match_edge, \
    model_service_graph_search_match_vertex, \
    model_service_graph_search_multi_hop_multi_edge_common_vertexes, \
    model_service_graph_search_find_path, \
    model_service_register_graph, \
    model_service_delete_graph, \
    model_service_show_graph, \
    model_service_insert_edge, \
    model_service_insert_vertex, \
    model_service_summary_graph, \
    model_service_description_graph, \
    model_service_path_finding, \
    model_service_create_subgraph, \
    model_service_update_subgraph_by_multi_hop_multi_edge, \
    model_service_update_subgraph_by_match_edge, \
    model_service_update_subgraph_by_find_path_multi_edge, \
    model_service_destroy_subgraph, \
    model_service_metric_indegree, \
    model_service_metric_outdegree, \
    model_service_metric_degree, \
    model_service_metric_pagerank, \
    model_service_edge_match_property, \
    model_service_vertex_match_property, \
    model_service_query_edges, \
    model_service_query_vertexes, \
    model_service_time_static_subgraph, \
    model_service_graph, \
    model_service_graph_insert, \
    model_service_graph_delete, \
    model_service_graph_update, \
    model_service_ga_build

from controllers.graph import GetInfoData,\
    GetHealthData, GaBuildHandler, GraphHandler, GraphInsertHandler,\
    GraphDeleteHandler, GraphUpdateHandler, GraphSearchMatchEdgeHandler,\
    GraphSearchMultiHopMultiEdgeCommonVertexesHandler, GraphSearchOneHopMultiEdgeHandler,\
    GraphSearchMatchVertexHandler, GraphSearchMultiHopMultiEdgeHandler,\
    RegisterGraphHandler,\
    DeleteGraphHandler, ShowGraphHandler, SummaryGraphHandler, DescriptionGraphHandler,\
    InsertEdgeHandler, InsertVertexHandler, HealthServiceHandler, InfoServiceHandler,\
    FindPathHandler, SubgraphCreationHandler, MultiHopSubgraphUpdateHandler,\
    PathSubgraphUpdateHandler, EdgeSubgraphUpdateHandler, SubgraphDestroyHandler,\
    MetricDegreeHandler, MetricIndegreeHandler, MetricPagerankHandler,\
    MetricOutdegreeHandler, VertexQueryHandler, VertexMatchPropertyHandler,\
    EdgeQueryHandler, EdgeMatchPropertyHandler, TimeStaticSubgraphHandler,\
    GainSubgraph, GainSubgraphQuery, GainTimeLineCount, GainCountEdges,\
    GainGraphTypeSet, GainGraphTypeFunction, GainGraphTypeCount, GainGraphTaskSelect, GainGraphTaskSubmit


logger = get_logger()


class DataServiceConfigLoad(object):
    def __init__(self):
        print("clickhouse config load start")

    def load_config(self):
        return load_config()


@register_swagger_model
class modelServiceMetricIndegree(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_metric_indegree(data, config_params)


@register_swagger_model
class model_serviceGABuild(object):
    def __init__(self):
        print("model_service_ga_build model start")

    def model_operation(self, data, config_params):
        return model_service_ga_build(data, config_params)

@register_swagger_model
class modelServiceGraph(object):
    def __init__(self):
        print("model_service_graph model start")

    def model_operation(self, data, config_params):
        return model_service_graph(data, config_params)


@register_swagger_model
class modelServiceGraphInsert(object):
    def __init__(self):
        print("model_service_graph_insert model start")

    def model_operation(self, data, config_params):
        return model_service_graph_insert(data, config_params)


@register_swagger_model
class modelServiceGraphDelete(object):
    def __init__(self):
        print("model_service_graph_delete model start")

    def model_operation(self, data, config_params):
        return model_service_graph_delete(data, config_params)

@register_swagger_model
class modelServiceGraphUpdate(object):
    def __init__(self):
        print("model_service_graph_update model start")

    def model_operation(self, data, config_params):
        return model_service_graph_update(data, config_params)


@register_swagger_model
class modelServiceGraphSearchOneHop(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_graph_search_one_hop(data, config_params)

@register_swagger_model
class modelServiceGraphSearchMultiHop(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_graph_search_multi_hop(data, config_params)

@register_swagger_model
class modelServiceGraphSearchMultiHopMultiEdge(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_graph_search_multi_hop_multi_edge(data, config_params)

@register_swagger_model
class modelServiceGraphSearchOneHopMultiEdge(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_graph_search_one_hop_multi_edge(data, config_params)

@register_swagger_model
class modelServiceGraphSearchMultiHopCommonVertexes(object):
    """
    # multi_hop_common_vertexes
    """
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_graph_search_multi_hop_common_vertexes(data, config_params)

# one pop multi edge
@register_swagger_model
class modelServiceGraphSearchMatchEdge(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_graph_search_match_edge(data, config_params)

# one pop multi edge
@register_swagger_model
class modelServiceGraphSearchMatchVertex(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_graph_search_match_vertex(data, config_params)

# one pop multi edge
@register_swagger_model
class modelServiceGraphSearchMultiHopMultiEdgeCommonVertexes(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_graph_search_multi_hop_multi_edge_common_vertexes(data, config_params)

# one pop multi edge
@register_swagger_model
class modelServiceGraphSearchMultiHopMultiEdgeCommonVertexes(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_graph_search_multi_hop_multi_edge_common_vertexes(data, config_params)

@register_swagger_model
class modelServiceGraphSearchFindPath(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_graph_search_find_path(data, config_params)

@register_swagger_model
class modelServiceRegisterGraph(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_register_graph(data, config_params)

@register_swagger_model
class modelServiceDeleteGraph(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_delete_graph(data, config_params)

@register_swagger_model
class modelServiceShowGraph(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_show_graph(config_params)

@register_swagger_model
class modelServiceInsertEdge(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_insert_edge(data, config_params)

@register_swagger_model
class modelServiceInsertVertex(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_insert_vertex(data, config_params)

@register_swagger_model
class modelServiceSummaryGraph(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_summary_graph(data, config_params)

@register_swagger_model
class modelServiceDescriptionGraph(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_description_graph(data, config_params)

@register_swagger_model
class modelServicePathFinding(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_path_finding(data, config_params)

@register_swagger_model
class modelServiceSubgraphCreation(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_create_subgraph(data, config_params)

@register_swagger_model
class modelServiceMultiHopSubgraphUpdate(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_update_subgraph_by_multi_hop_multi_edge(data, config_params)

@register_swagger_model
class modelServicePathSubgraphUpdate(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_update_subgraph_by_find_path_multi_edge(data, config_params)

@register_swagger_model
class modelServiceEdgeSubgraphUpdate(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_update_subgraph_by_match_edge(data, config_params)

@register_swagger_model
class modelServiceSubgraphDestroy(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_destroy_subgraph(data, config_params)

@register_swagger_model
class modelServiceMetricOutdegree(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_metric_outdegree(data, config_params)

@register_swagger_model
class modelServiceMetricDegree(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_metric_degree(data, config_params)

@register_swagger_model
class modelServiceMetricPagerank(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_metric_pagerank(data, config_params)

@register_swagger_model
class modelServiceEdgeMatchProperty(object):
    def __init__(self):
        print("query edge prop model start")

    def model_operation(self, data, config_params):
        return model_service_edge_match_property(data, config_params)

@register_swagger_model
class modelServiceVertexMatchProperty(object):
    def __init__(self):
        print("query vertex prop model start")

    def model_operation(self, data, config_params):
        return model_service_vertex_match_property(data, config_params)

@register_swagger_model
class modelServiceQueryEdge(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_query_edges(data, config_params)

@register_swagger_model
class modelServiceQueryVertex(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_query_vertexes(data, config_params)

@register_swagger_model
class modelServiceTimeStaticSubgraph(object):
    def __init__(self):
        print("clickhouse model start")

    def model_operation(self, data, config_params):
        return model_service_time_static_subgraph(data, config_params)


class Application(tornado.web.Application):
    data_service = DataServiceConfigLoad()

    config_params = data_service.load_config()

    model_serviceGABuild = model_serviceGABuild()

    model_serviceGraphInsert = modelServiceGraphInsert()

    model_serviceGraphDelete = modelServiceGraphDelete()

    model_serviceGraphUpdate = modelServiceGraphUpdate()

    model_serviceGraph = modelServiceGraph()

    model_serviceGraphSearchOnePop = modelServiceGraphSearchOneHop()

    model_serviceGraphSearchMultiHopMultiEdge = modelServiceGraphSearchMultiHopMultiEdge()

    model_serviceGraphSearchMultiHop = modelServiceGraphSearchMultiHop()

    model_serviceGraphSearchOneHopMultiEdge = modelServiceGraphSearchOneHopMultiEdge()

    model_serviceGraphSearchMultiHopCommonVertexes = modelServiceGraphSearchMultiHopCommonVertexes()

    model_serviceGraphSearchMatchEdge = modelServiceGraphSearchMatchEdge()

    model_serviceGraphSearchMatchVertex = modelServiceGraphSearchMatchVertex()

    model_serviceGraphSearchMultiHopMultiEdgeCommonVertexes = modelServiceGraphSearchMultiHopMultiEdgeCommonVertexes()

    model_serviceGraphSearchFindPath = modelServiceGraphSearchFindPath()

    model_serviceRegisterGraph = modelServiceRegisterGraph()

    model_serviceDeleteGraph = modelServiceDeleteGraph()

    model_serviceShowGraph = modelServiceShowGraph()

    model_serviceInsertEdge = modelServiceInsertEdge()

    model_serviceInsertVertex = modelServiceInsertVertex()

    model_serviceSummaryGraph = modelServiceSummaryGraph()

    model_serviceDescriptionGraph = modelServiceDescriptionGraph()

    model_serviceFindPath = modelServicePathFinding()

    model_serviceSubgraphCreation = modelServiceSubgraphCreation()

    model_serviceMultiHopSubgraphUpdate = modelServiceMultiHopSubgraphUpdate()

    model_servicePathSubgraphUpdate = modelServicePathSubgraphUpdate()

    model_serviceEdgeSubgraphUpdate = modelServiceEdgeSubgraphUpdate()

    model_serviceSubgraphDestroy = modelServiceSubgraphDestroy()

    model_serviceMetricIndegree = modelServiceMetricIndegree()

    model_serviceMetricOutdegree = modelServiceMetricOutdegree()

    model_serviceMetricDegree = modelServiceMetricDegree()

    model_serviceMetricPagerank = modelServiceMetricPagerank()

    model_serviceEdgeMatchProperty = modelServiceEdgeMatchProperty()

    model_serviceVertexMatchProperty = modelServiceVertexMatchProperty()

    model_serviceQueryEdge = modelServiceQueryEdge()

    model_serviceQueryVertex = modelServiceQueryVertex()

    model_serviceTimeStaticSubgraph = modelServiceTimeStaticSubgraph()

    _routes = [
        tornado.web.url(r"/chgraph/api/v1/info", GetInfoData),
        tornado.web.url(r"/chgraph/api/v1/health", GetHealthData),
        tornado.web.url(r"/graph-db/api/v1/ga-build", GaBuildHandler,
                        {"model_service": model_serviceGABuild, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/graph", GraphHandler,
                        {"model_service": model_serviceGraph, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/graph-insert", GraphInsertHandler,
                        {"model_service": model_serviceGraphInsert, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/graph-delete", GraphDeleteHandler,
                        {"model_service": model_serviceGraphDelete, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/graph-update", GraphUpdateHandler,
                        {"model_service": model_serviceGraphUpdate, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/multi-hop", GraphSearchMultiHopMultiEdgeHandler,
                        {"model_service": model_serviceGraphSearchMultiHopMultiEdge, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/one-hop", GraphSearchOneHopMultiEdgeHandler,
                        {"model_service": model_serviceGraphSearchOneHopMultiEdge, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/edges", GraphSearchMatchEdgeHandler,
                        {"model_service": model_serviceGraphSearchMatchEdge, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/vertices", GraphSearchMatchVertexHandler,
                        {"model_service": model_serviceGraphSearchMatchVertex, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/multi-hop-common-vertices",
                        GraphSearchMultiHopMultiEdgeCommonVertexesHandler,
                        {"model_service": model_serviceGraphSearchMultiHopMultiEdgeCommonVertexes,
                         "config_params": config_params}),
        # tornado.web.url(r"/graph-db/api/v1/paths", graphSearchFindPathHandler,{"model_service": model_serviceGraphSearchFindPath,"config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/graph-registration", RegisterGraphHandler,
                        {"model_service": model_serviceRegisterGraph, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/graph-deletion", DeleteGraphHandler,
                        {"model_service": model_serviceDeleteGraph, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/graph-show", ShowGraphHandler,
                        {"model_service": model_serviceShowGraph, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/graph-summary/(?P<graph_name>\S*)", SummaryGraphHandler,
                        {"model_service": model_serviceSummaryGraph, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/graph-description/(?P<graph_name>\S*)", DescriptionGraphHandler,
                        {"model_service": model_serviceDescriptionGraph, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/edge-insertion", InsertEdgeHandler,
                        {"model_service": model_serviceInsertEdge, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/vertex-insertion", InsertVertexHandler,
                        {"model_service": model_serviceInsertVertex, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/health", HealthServiceHandler),
        tornado.web.url(r"/graph-db/api/v1/info", InfoServiceHandler),
        tornado.web.url(r"/graph-db/api/v1/path", FindPathHandler,
                        {"model_service": model_serviceFindPath, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/subgraph-creation", SubgraphCreationHandler,
                        {"model_service": model_serviceSubgraphCreation, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/subgraph-update-multi-hop", MultiHopSubgraphUpdateHandler,
                        {"model_service": model_serviceMultiHopSubgraphUpdate, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/subgraph-update-path", PathSubgraphUpdateHandler,
                        {"model_service": model_servicePathSubgraphUpdate, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/subgraph-update-edge", EdgeSubgraphUpdateHandler,
                        {"model_service": model_serviceEdgeSubgraphUpdate, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/subgraph-destruction", SubgraphDestroyHandler,
                        {"model_service": model_serviceSubgraphDestroy, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/metric-indegree", MetricIndegreeHandler,
                        {"model_service": model_serviceMetricIndegree, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/metric-outdegree", MetricOutdegreeHandler,
                        {"model_service": model_serviceMetricOutdegree, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/metric-degree", MetricDegreeHandler,
                        {"model_service": model_serviceMetricDegree, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/metric-pagerank", MetricPagerankHandler,
                        {"model_service": model_serviceMetricPagerank, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/vertex-property", VertexMatchPropertyHandler,
                        {"model_service": model_serviceVertexMatchProperty, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/edge-property", EdgeMatchPropertyHandler,
                        {"model_service": model_serviceEdgeMatchProperty, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/vertex-query", VertexQueryHandler,
                        {"model_service": model_serviceQueryVertex, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/edge-query", EdgeQueryHandler,
                        {"model_service": model_serviceQueryEdge, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/subgraph-time-statistic", TimeStaticSubgraphHandler,
                        {"model_service": model_serviceTimeStaticSubgraph, "config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/subgraph-filtration-statistic", GainSubgraph,
                        {"config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/time-line-count", GainTimeLineCount,
                        {"config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/round-edges-count", GainCountEdges,
                        {"config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/subgraph-query", GainSubgraphQuery,
                        {"config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/subgraph-type-statistics", GainGraphTypeCount,
                        {"config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/subgraph-type-statistics-set", GainGraphTypeSet,
                        {"config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/subgraph-type-statistics-function", GainGraphTypeFunction,
                        {"config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/subgraph-calculation-select", GainGraphTaskSelect,
                        {"config_params": config_params}),
        tornado.web.url(r"/graph-db/api/v1/subgraph-calculation-submit", GainGraphTaskSubmit,
                        {"config_params": config_params}),
    ]

    def get_routes(self):
        return self._routes

    def __init__(self):
        settings = {"debug": True}
        # setup_swagger(self._routes,
        #               swagger_url='/doc',
        #               api_base_url='/',
        #               description='',
        #               api_version='1.0.0',
        #               title='Journal API',
        #               contact='name@domain',
        #               schemes=['https'],
        #               security_definitions={
        #                   'ApiKeyAuth': {
        #                       'type': 'apiKey',
        #                       'in': 'header',
        #                       'name': 'X-API-Key'
        #                   }
        #               })
        super(Application, self).__init__(self._routes, **settings)


def main():
    App = Application()
    App.get_routes()
    App.listen(10110)
    tornado.ioloop.IOLoop.current().start()


if __name__ == "__main__":
    main()



import datetime

from config.logger_design import get_logger

logger = get_logger()


def get_graph_vertexes(graph_cfg, graph):
    vertexes = []
    i = 0
    for vertex, vertex_info in graph_cfg.get("vertexes").items():
        i += 1

        graph_node = {}
        selected_fields = []
        for key in ["id", "label"]:
            selected_fields.append(vertex_info.get(key))

        selected_fields.extend(vertex_info.get("fields") or [])
        start = datetime.datetime.now()
        try:
            vertex_query = graph.query_vertexes(vertex, [""], selected_fields, "df")
            end = datetime.datetime.now()
            logger.info("query_vertexes {} count_times: {}".format(i, round((end - start).total_seconds(), 5) * 1000))
        except Exception as e:
            raise Exception("vertex query failed: {}".format(str(e)))

        graph_node["type"] = vertex
        graph_node["data"] = vertex_query

        vertexes.append(graph_node)

    return vertexes


def get_graph_edges(graph_cfg, graph):
    edges = []
    i = 0
    for edge, edge_info in graph_cfg.get("edges").items():
        i += 1

        graph_edge = {}
        selected_fields = []
        for key in ["src", "dst", "rank"]:
            selected_fields.append(edge_info.get(key))
        selected_fields.extend(edge_info.get("fields") or [])
        start = datetime.datetime.now()
        try:
            edges_result = graph.query_edges(edge, [""], selected_fields, "df")
            end = datetime.datetime.now()
            logger.info("query_edge {} count_times: {}".format(i, round((end - start).total_seconds(), 5) * 1000))
        except Exception as e:
            raise Exception("edge query failed: {}".format(str(e)))

        graph_edge["type"] = edge
        graph_edge["data"] = edges_result
        graph_edge["id"] = edge

        edges.append(graph_edge)

    return edges



import os
import sys

PARENT_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(PARENT_DIR)

from celery import Celery

app = Celery('ga_chgraph')
app.config_from_object("celery_tasks.celery_config")




from config import logger_design

logger = logger_design.get_logger()


class CHGraph(object):

    def __init__(self, client):
        self.client = client
        logger.info('CHGraph Start')

    def execute(self, sql):
        res = self.client.query_dataframe(sql)
        return res



import datetime


def log_execute_time(logger):
    def _log_execute_time(f):
        def __log_execute_time(*args, **kwargs):
            start = datetime.datetime.now()
            rest = f(*args, **kwargs)
            end = datetime.datetime.now()
            logger.info("function {} execute total cost {}.".format(f.__name__,
                                                                    round((end - start).total_seconds(), 5) * 1000))
            return rest

        return __log_execute_time

    return _log_execute_time



"""
拼接sql
"""


class SqlMergeHandler(object):
    def __init__(self, graph_type, subgraph_name, edge_or_vertex_info, cluster_name=''):
        """
        graph_type: 边表还是顶点表
        is_distributed_table: true/false  分区表/本地表
        """
        self.graph_type = graph_type
        self.edge_or_vertex_info = edge_or_vertex_info
        self.cluster_name = cluster_name
        self.subgraph_name = subgraph_name

        self.table_name = self.__get_table_name_from_graph_config()  # 分区表的表名
        self.local_table_name = self.__get_local_table_name()  # 本地表的表名
        self.order_by_fields = self.__get_order_by_fields()

        if not self.table_name or not self.local_table_name:
            raise ValueError("SqlMergeHandler init failed")

        if not cluster_name:
            raise ValueError("cluster name is NULL")

    def __get_table_name_from_graph_config(self):
        table_name = self.edge_or_vertex_info.get("table")
        return "{}.{}".format(self.subgraph_name, table_name)

    def __get_order_by_fields(self):
        if self.graph_type == "edges":
            keys = ["src", "dst", "rank"]
        else:
            keys = ["id"]

        order_by_fields = [self.edge_or_vertex_info.get(item) for item in keys]
        return order_by_fields

    def __get_local_table_name(self):
        return self.table_name + '_shard'

    def get_table_fields_and_types_from_schema(self, table_schema_str):
        """
        获取表的所有字段和字段类型
        """
        start, end = 0, 0
        sentences = table_schema_str.split('\n')
        for idx, item in enumerate(sentences):
            field_types = item.split()
            if len(field_types) >= 2:
                field_name = field_types[0]
                field_type = field_types[1]
                if 'Nullable' in field_type:
                    field_name = field_name.replace("`", "")
                    if field_name in self.order_by_fields:
                        new_field_type = field_type.replace("Nullable(", "").replace(")", "")
                        field_types[1] = new_field_type
                        new_item = ' '.join(field_types)
                        sentences[idx] = new_item

            if item == "(":
                start = idx

            if item == ")":
                end = idx

            if item.startswith("ENGINE"):
                break

        fields_and_types_str = ''.join(sentences[start:end + 1])
        return fields_and_types_str

    # def get_table_fields_and_types(self):
    #     """
    #     封装字段和类型
    #     """
    #     sql_schema = "(\n    `id` Nullable(String),\n   " \
    #     " `task_id` Nullable(String),\n    " \
    #     "`graph_algorithm_type` Nullable(String),\n   " \
    #     " `val` Nullable(String),\n    " \
    #     "`create_time` DateTime64(3) DEFAULT now() CODEC(DoubleDelta, LZ4)\n)\n"

        # return sql_schema

    def get_cluster_table_engine(self, is_distributed_table):
        """
        Distributed(cluster_name, database, table, [sharding_key])
        """
        engine_str = "ENGINE = "
        names = self.local_table_name.split(".")
        table_name = names[1]
        if not is_distributed_table:
            engine_str += "ReplicatedMergeTree('/clickhouse/tables/" + self.local_table_name + "/{shard}', " \
                                                                                               "'{replica}')"
        else:
            engine_str += "Distributed(" + self.cluster_name + ", " + self.subgraph_name + ", " + table_name \
                + ", rand())"

        return engine_str

    def get_single_table_engine(self):
        """
        single engine
        """
        engine_str = "ENGINE = MergeTree"
        return engine_str

    def get_table_order_by_fields(self):
        order_by_fields = self.order_by_fields
        return " order by ({})".format(",".join(order_by_fields))

    def merge_sql_create_table(self, is_distributed_table, columns_str, engine_str, order_by_str):
        """
        合并建表语句
        """
        if not is_distributed_table:
            sql = "create table if not exists " + self.local_table_name + " on cluster " + self.cluster_name + columns_str + \
                  engine_str + order_by_str + " SETTINGS index_granularity = 8192"
        else:
            sql = "create table if not exists " + self.table_name + " on cluster " + self.cluster_name + ' as ' + \
                  self.local_table_name + " " + engine_str

        return sql


    def merge_single_sql_create_table(self, columns_str, engine_str, order_by_str):
        """
        合并建表语句
        """
        sql = "create table if not exists " + self.table_name + columns_str + \
               engine_str + order_by_str + " SETTINGS index_granularity = 8192"
        return sql


    def gen_create_table_sql(self, table_schema_str, is_distributed_table=False):
        """
        拼接集群sql
        """
        columns_str = self.get_table_fields_and_types_from_schema(table_schema_str)
        engine_str = self.get_cluster_table_engine(is_distributed_table)
        order_by_str = self.get_table_order_by_fields()
        sql = self.merge_sql_create_table(is_distributed_table, columns_str, engine_str, order_by_str)
        return sql

    def gen_create_new_table_sql(self, table_schema_str, is_distributed_table=False):
        """
        拼接集群sql
        """
        columns_str = self.get_table_fields_and_types_from_schema(table_schema_str)
        engine_str = self.get_cluster_table_engine(is_distributed_table)
        order_by_str = " order by (id)"
        sql = self.merge_sql_create_table(is_distributed_table, columns_str, engine_str, order_by_str)
        return sql




if __name__ == "__main__":
    sql_schema = "CREATE TABLE sub_graph_test_2.default_metagroups\n(\n    `group_id` Nullable(String),\n   " \
                 " `group_name` Nullable(String),\n    `num_members` Nullable(Int64),\n    " \
                 "`category_id` Nullable(String),\n    `category_name` Nullable(String),\n   " \
                 " `organizer_id` Nullable(String),\n    `group_urlname` Nullable(String),\n    " \
                 "`_time` DateTime64(3) DEFAULT now() CODEC(DoubleDelta, LZ4)\n)\nENGINE = MergeTree\n" \
                 "ORDER BY _time\nSETTINGS index_granularity = 8192"

    info = {
            "db":"sub_graph_test_2",
            "table":"default_metagroups",
            "id":"group_id",
            "label":"group_name",
            "type":"group",
            "id_data_type":"String",
            "fields":[
                "group_name",
                "num_members",
                "category_id",
                "category_name",
                "organizer_id",
                "group_urlname"
            ],
            "types":[
                "String",
                "Num",
                "String",
                "String",
                "String",
                "String"
            ]
        }
    handler = SqlMergeHandler("vertexes", "subgraph_test_11", info,
                              cluster_name="cluster_2shards_2replicas")
    sql = handler.gen_create_table_sql(sql_schema)
    print("local table sql: {}\n".format(sql))

    sql = handler.gen_create_table_sql(sql_schema, True)
    print("distribute table sql: {}\n".format(sql))
#     """-- auto-generated definition
# create table xiqianedge
# (
#     record_date    DateTime64(3),
#     record_time    DateTime64(3),
#     tran_id        Nullable(String),
#     orig_acct      Nullable(String),
#     bene_acct      Nullable(String),
#     tx_type        Nullable(String),
#     base_amt       Nullable(Float32),
#     tran_timestamp Nullable(String),
#     is_sar         Nullable(String),
#     alert_id       Nullable(String),
#     _time          DateTime64(3) alias record_time
# )
#     engine = MergeTree ORDER BY record_date
#         SETTINGS index_granularity = 8192;"""




import re
"""
替换sql
"""


class SqlReplaceHandler(object):
    def __init__(self, is_cluster=False):
        self.is_cluster = is_cluster

    @staticmethod
    def replace_in_with_global_in(origin_sql):
        """
        sql中的关键字必须小写
        """
        pattern = r" in \( *select +"
        return re.sub(pattern, " global in (select ", origin_sql)

    @staticmethod
    def replace_join_with_global_join(origin_sql):
        patterns = [
            (r"( inner join \( *select )|( join \( *select )", " global join (select "),

            (r"( left outer join \( *select )|( left join \( *select )|( left outer global join \( *select )|"
             r"( left global join \( *select )",  " global left join (select "),

            (r"( right outer join \( *select )|( right join \( *select )|( right outer global join \( *select )|"
             r"( right global join \( *select )", " global right join (select "),

            (r"( cross join \( *select )|( cross global join \( *select )", " global cross join (select "),

        ]

        for pattern, repl in patterns:
            origin_sql = re.sub(pattern, repl, origin_sql)

        return origin_sql

    def deal_sql(self, origin_sql):
        if not self.is_cluster:
            return origin_sql

        if re.search(r" in \( *select +", origin_sql):
            sql = self.replace_in_with_global_in(origin_sql)
        else:
            sql = origin_sql

        if "join" in sql:
            sql = self.replace_join_with_global_join(sql)
        else:
            sql = origin_sql
        return sql


if __name__ == "__main__":
    handler = SqlReplaceHandler(True)

    sql_1 = "insert into default.user_record (*) select * from login_record\
                  where user_id in ( select user_id from login_record where user_id>8)"
    new_sql = handler.deal_sql(sql_1)
    print("new sql: ", new_sql)

    sql_2 = "insert into default.user_record (*) select * from login_record\
                      where user_id in ('select', 'a', 'b')"
    new_sql = handler.deal_sql(sql_2)
    print("new sql: ", new_sql)

    sql_3 = "select * from default.login_record l inner join (select * from default.user_record) u on l.user_id=u.user_id;"
    sql_3 = handler.deal_sql(sql_3)
    print("sql3: {}\n".format(sql_3))

    sql_4 = "select * from default.login_record l left outer join (select * from default.user_record) u on l.user_id=u.user_id;"
    sql_4 = handler.deal_sql(sql_4)
    print("sql4: {}\n".format(sql_4))

    sql_5 = "select l.* from default.default_metamembers as l join (select member_id id from (SELECT * FROM default_mergeedge limit 30  )) as r on l.member_id=r.id"
    sql_5 = handler.deal_sql(sql_5)
    print("sql5: {}\n".format(sql_5))



"""
plato调用客户端
"""
import os
import sys

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

import datetime
import subprocess
import pandas as pd
import psycopg2

from config.logger_design import get_logger
from config.config import plato_config
from config.data_load_config import load_config

from utils.common import unzip_file

logger = get_logger()


class PlatoException(Exception):
    pass


class PlatoClient(object):
    def __init__(self, sub_graph, task_id, calculation_type):
        """
        :param sub_graph: 子图名称
        """
        global_config = load_config()
        db = global_config.get("db")
        ck_client = global_config.get("graphClient")

        self.sub_graph = sub_graph
        self.db = db
        self.ck_client = ck_client
        self.task_id = task_id
        self.calculation_type = calculation_type
        self.current_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
        self.output_file_dir = self.get_output_file_dir()

    def get_sub_graph_config(self):
        graph_config = self.db.use_tables(self.sub_graph)
        self.sub_graph_config = graph_config

    def get_output_file_dir(self):
        parent_dir = self.current_dir
        output_file_dir = os.path.join(parent_dir, 'output/')
        if not os.path.exists(output_file_dir):
            os.mkdir(output_file_dir)

        return output_file_dir

    @staticmethod
    def gen_calculation_config(cal_type, input_file, output_file):
        """
        图算法配置
        """
        calculation_config = {
            "Pagerank": {
                "param_list": ["ROOT_DIR", "WNUM", "WCORES", "INPUT", "OUTPUT", "IS_DIRECTED",
                              "EPS", "DAMPING", "ITERATIONS"],
                "value_list": [plato_config.plato_install_path, "4", "4", input_file, output_file, "false",
                               "0.0001", "0.85", "100"],
                "shell": "run_pagerank_local.sh"
            },
            "Betweenness": {
                "param_list": ["ROOT_DIR", "WNUM", "WCORES", "INPUT", "OUTPUT", "IS_DIRECTED",
                               "ALPHA", "PART_BY_IN", "CHOSEN", "MAX_ITERATION", "CONSTANT"],
                "value_list": [plato_config.plato_install_path, "4", "4", input_file, output_file, "false",
                               "-1", "false", "-1", "0", "2"],
                "shell": "run_bnc_local.sh"
            },
            "Closeness": {
                "param_list": ["ROOT_DIR", "WNUM", "WCORES", "INPUT", "OUTPUT", "IS_DIRECTED",
                               "ALPHA", "PART_BY_IN", "NUM_SAMPLES"],
                "value_list": [plato_config.plato_install_path, "4", "4", input_file, output_file, "false",
                               "-1", "false", "10"],
                "shell": "run_cnc_local.sh"
            }
        }

        return calculation_config.get(cal_type)

    def get_edge_infos(self):
        edges = []
        edge_config = self.sub_graph_config.get("edges") or {}
        for edge_name, info in edge_config.items():
            edges.append(info)
        self.edges = edges

    def get_vertex_infos(self):
        vertex_config = self.sub_graph_config.get("vertexes") or {}
        self.vertexes = vertex_config

    def get_vertexes_data(self, table_name, field_name):
        """
        从clickhouse中获取顶点数据
        """
        logger.info("[PlatoClient]-[get_vertexes_data]  table_name:{}    field_name:{}".format(table_name,
                                                                                        field_name))
        sql = "select distinct " + field_name + " from " + table_name + " order by " + field_name
        df = self.ck_client.query_dataframe(sql)
        if df.empty:
            return []
        columns = df.columns.tolist()
        vertexs = df[columns[0]].tolist()
        return vertexs

    @staticmethod
    def mapping_vertexes(start=0, vertexes=None):
        """
        处理顶点数据
        """
        vertexes_map = {}
        for item in vertexes or []:
            vertexes_map[item] = start
            start += 1
        return start, vertexes_map

    def get_edges_data(self, table_name, field_names):
        field_names_str = ','.join(field_names)
        sql = "select distinct " + field_names_str + " from " + table_name
        logger.info("[get_edges_data] sql: {}".format(sql))

        df = self.ck_client.query_dataframe(sql)
        if df.empty:
            return []

        data = []
        df = df.sort_values(by=field_names)
        df = df[field_names[0:2]]
        for record in df.to_records(index=False):
            data.append(record.tolist())

        return data

    @staticmethod
    def mapping_edges(edges, vertexes_map, vertex_types):
        if list(vertexes_map.keys()) != vertex_types:
            logger.error("映射边数据错误: vertex_types:{}   vertexes_map.keys:{} ".format(vertex_types,
                                                                    vertexes_map.keys()))
            raise PlatoException("映射边数据错误.")
        data = []
        for (src, dist) in edges:
            src_idx = vertexes_map.get(vertex_types[0]).get(src)    # 注意df中src的指的类型
            if src_idx is None:
                raise PlatoException("边数据替换报错：起点{}找不到对应的下标".format(src))
            dist_idx = vertexes_map.get(vertex_types[0]).get(dist) if len(vertex_types) == 1 else\
                                           vertexes_map.get(vertex_types[1]).get(dist)
            if dist_idx is None:
                raise PlatoException("边数据替换报错：终点{}找不到对应的下标".format(dist))

            data.append([src_idx, dist_idx])

        return data

    def get_vertexes_mapping_data(self, edge_info):
        """
        获取映射后的顶点数据
        """
        vertexes_map = {}
        db_name = edge_info.get("db")
        src_type = edge_info.get("src_type")
        dst_type = edge_info.get("dst_type")
        if src_type == dst_type:
            vertex_types = ["src_type"]
        else:
            vertex_types = ["src_type", "dst_type"]

        start = 0
        for _type in vertex_types:
            vertex_type = edge_info.get(_type)
            if vertex_type not in vertexes_map:
                vertexes_map[vertex_type] = {}

            vertex_table_name = self.vertexes.get(vertex_type, {}).get("table")
            full_table_name = "{}.{}".format(db_name, vertex_table_name)
            src_field_name = self.vertexes.get(vertex_type, {}).get("id")

            vertexes_data = self.get_vertexes_data(full_table_name, src_field_name)
            _start, vertexes_idx_data = self.mapping_vertexes(start, vertexes_data)
            start = _start
            vertexes_map[vertex_type].update(vertexes_idx_data)

        return vertexes_map

    def get_edge_mapping_data(self, edge_info, vertex_idx_data):
        """
        把边的定点数据进行映射
        :param edge_info: {"dst":"", "src":"", "table":"", "db":""}
        :param vertex_idx_data: {"group":{"定点ID"：0， "定点ID"：1， "定点ID"：2}, "friend":{}}
        """
        db_name = edge_info.get("db")
        table_name = edge_info.get("table")
        table_name = "{}.{}".format(db_name, table_name)

        field_names = []
        for field in ["src", "dst", "rank"]:
            field_names.append(edge_info.get(field))

        vertex_types = []
        for field in ["src_type", "dst_type"]:
            val = edge_info.get(field)
            if val not in vertex_types:
                vertex_types.append(val)

        edge_data = self.get_edges_data(table_name, field_names)
        edge_idx_data = self.mapping_edges(edge_data, vertex_idx_data, vertex_types)
        return edge_idx_data

    def write_to_csv(self, edge_data):
        """
        把映射后的边数据写入到csv文件中，只包含起点和终点
        """
        output_file_name = "{}_{}_input.csv".format(str(self.task_id).replace("-", ""),
                                                    self.calculation_type.lower())
        output_file_path = os.path.join(self.output_file_dir, output_file_name)
        df = pd.DataFrame(edge_data)
        df.to_csv(output_file_path, header=False, index=False)

        logger.info("write to csv success !!!  file_path:{}".format(output_file_path))
        return output_file_path

    def execute_plato(self, edge_info_file):
        """
        调用plato
        """
        output_file_path = os.path.join(self.output_file_dir, '{}/'.format(self.calculation_type))
        cal_config = self.gen_calculation_config(self.calculation_type, edge_info_file,
                                                 output_file_path)
        cal_param_val = cal_config.get("value_list")
        cal_sh_dir = os.path.join(self.current_dir, "shells/plato_shell/")
        cmds = ["bash", "{}{}".format(cal_sh_dir, cal_config.get("shell"))]
        cmds.extend(cal_param_val)
        logger.info("[execute_plato] commands: {}".format(cmds))

        res = subprocess.run(cmds, stderr=subprocess.PIPE, stdout=subprocess.PIPE)
        if res.returncode == 0:
            return True, "", output_file_path

        return False, res.stderr, output_file_path

    @staticmethod
    def read_from_csv(file_path):
        file_name = "0003_0000.csv.gz"
        if not os.path.exists(os.path.join(file_path, file_name)):
            raise PlatoException("在{}没有找到文件{}".format(file_path, file_name))
        else:
            unzip_file_path = unzip_file(file_path, file_name)
        df = pd.read_csv(unzip_file_path, names=['A', 'B'])
        data = []
        for record in df.to_records(index=False):
            data.append(record.tolist())

        return data

    def write_to_ck(self, vertex_cal_data, vertex_idx_data):
        """
        将图计算结果写入到clickhouse
        """
        graph_cal_res_table_name = "calculation_attribute"
        idx_vertex_data = {}
        for _type, info in vertex_idx_data.items():
            for vertex_id, idx in info.items():
                idx_vertex_data[idx] = [vertex_id, _type]

        insert_data = []
        table_name = "{}.{}".format(self.sub_graph, graph_cal_res_table_name)
        for (vertex_id, val) in vertex_cal_data:
            new_vertex_id, vertex_type = idx_vertex_data.get(vertex_id)
            if new_vertex_id is None:
                raise PlatoException("写入clickhouse失败. {}对应的顶点不存在".format(vertex_id))
            record = [str(self.task_id), self.calculation_type, str(new_vertex_id), vertex_type, float(val),
                      datetime.datetime.now()]
            insert_data.append(record)

        field_names = "(task_id,graph_algorithm_type,id,id_type, val, create_time)"
        sql = "insert into " + table_name + field_names + " values "
        logger.info("[write_to_ck]  sql: {}".format(sql))
        self.ck_client.execute(sql, insert_data)

    def change_graph_cal_task_status(self, task_status):
        """
        修改图算法对应的任务的状态
        """
        task_table_name = "calculation_state"
        sql = "update " + task_table_name + " set operation_state=" + str(task_status) + "," + \
              "update_time=current_timestamp() where id=" + str(self.task_id) + ";"
        logger.info("[change_graph_cal_task_status] sql:{}".format(sql))

        conn = psycopg2.connect(self.db.url)
        logger.info("[change_graph_cal_task_status] self.db.url:{}".format(self.db.url))

        with conn.cursor() as cur:
            cur.execute(sql)
            conn.commit()

        conn.close()

    def run(self):
        try:
            self.get_sub_graph_config()
            self.get_edge_infos()
            self.get_vertex_infos()

            if not self.edges:
                raise PlatoException("子图的边类型配置为空. edge_config:{}".format(self.sub_graph_config))

            for edge_info in self.edges:
                vertex_idx_data = self.get_vertexes_mapping_data(edge_info)
                edge_idx_data = self.get_edge_mapping_data(edge_info, vertex_idx_data)
                edge_data_file = self.write_to_csv(edge_idx_data)
                success, err_msg, output_file_path = self.execute_plato(edge_data_file)
                if not success:
                    logger.error("执行图计算shell脚本失败: {}".format(err_msg))
                    raise PlatoException("执行图计算shell脚本失败.")

                vertex_cal_data = self.read_from_csv(output_file_path)
                self.write_to_ck(vertex_cal_data, vertex_idx_data)
        except Exception as exc:
            logger.error("调用plato失败：{}".format(str(exc)))
            success = 2      # 失败
        else:
            success = 1      # 成功
        finally:
            if self.ck_client:
                self.ck_client.disconnect()      # 关闭链接

        self.change_graph_cal_task_status(success)

        logger.info("execute plato completed!!!  task_id:{} ".format(self.task_id))


if __name__ == "__main__":
    client = PlatoClient("7cf53539_81d6_4986_9745_86d2acfc97c6", 7, 'pagerank')
    print("subgraph config: {}\n".format(client.sub_graph_config))

    client.get_edge_infos()
    print("client.edges: {}\n".format(client.edges))

    for edge_info in client.edges:
        print("edge_info:{}\n".format(edge_info))

        vertex_idx_data = client.get_vertexes_mapping_data(edge_info)
        print("vertex_idx_data: {}\n".format(vertex_idx_data))

        edge_idx_data = client.get_edge_mapping_data(edge_info, vertex_idx_data)
        print("edge_idx_data: {}\n".format(edge_idx_data))

        edge_data_file = client.write_to_csv(edge_idx_data)
        success, err_msg, output_file_path = client.execute_plato(edge_data_file)
        print("success:{}    err_msg:{}        output_file_path:{}".format(success, err_msg,
                                                                           output_file_path))
        # output_file_path = os.path.join(client.output_file_dir, 'pagerank/')
        print("output_file_path: ", output_file_path)

        vertex_cal_data = client.read_from_csv(output_file_path)
        print("vertex_cal_data: {}\n".format(vertex_cal_data))

        client.write_to_ck(vertex_cal_data, vertex_idx_data)

        client.change_graph_cal_task_status(1)





import os

broker_url = os.getenv("CELERY_BROKER_URL") or 'redis://localhost:6379/0'
# result_backend = os.getenv("CELERY_BACKEND_URL") or 'redis://localhost:6379/1'

task_serializer = 'json'
result_serializer = 'json'
accept_content = ['json']
timezone = 'Asia/Shanghai'
enable_utc = False

imports = ["celery_tasks.tasks"]


import os


class ClusterConfig:
    def __init__(self):
        self.cluster_name = None
        self.is_cluster = False
        self.__get_cluster_setting()

    def __get_cluster_setting(self):
        cluster_name = os.getenv("DAISY_CLUSTER_NAME", "")
        is_cluster = True if cluster_name else False

        self.cluster_name = cluster_name
        self.is_cluster = is_cluster


class PlatoConfig(object):
    def __init__(self):
        self.plato_install_path = os.getenv("PLATO_PATH", "/opt/app-root/src/plato")


plato_config = PlatoConfig()



import os
import ujson
from utils.ck_client import get_client
from daos.graph_op import CHGraph
from daos.db_op import DBoperator
from config.logger_design import get_logger

logger = get_logger()


def load_config():
    print("服务所有配置开始加载")
    parent_dir = os.path.dirname(os.path.abspath(__file__))
    clickhouse_config_dir = os.path.join(parent_dir, "graph_config.json")

    with open(clickhouse_config_dir, 'r') as f:
        clickhouse_config = ujson.load(f)

    # clickhouse config
    clickhouse_ip = os.getenv("HOST_DAISY") if os.getenv("HOST_DAISY") else clickhouse_config["ip"]
    clickhouse_port = os.getenv("PORT_TCP_DAISY") if os.getenv("PORT_TCP_DAISY") else clickhouse_config["port"]
    clickhouse_user = os.getenv("USERNAME_DAISY") if os.getenv("USERNAME_DAISY") else clickhouse_config["user"]
    clickhouse_pwd = os.getenv("PASSWORD_DAISY") if os.getenv("PASSWORD_DAISY") else clickhouse_config["password"]
    print(clickhouse_ip, clickhouse_port, clickhouse_user, clickhouse_pwd)
    logger.info("clickhouse_ip is: " + clickhouse_ip)

    # rocksdb config
    db_ip = os.getenv("DB_HOST") if os.getenv("DB_HOST") else clickhouse_config["dbip"]
    db_port = os.getenv("DB_PORT") if os.getenv("DB_PORT") else clickhouse_config["dbport"]
    db_user = os.getenv("DB_USERNAME") if os.getenv("DB_USERNAME") else clickhouse_config["dbuser"]
    db_pwd = os.getenv("DB_PASSWORD") if os.getenv("DB_PASSWORD") else clickhouse_config["dbpassword"]
    db_table = os.getenv("DB_NAME") if os.getenv("DB_NAME") else clickhouse_config["dbtable"]
    db_ssl_mode = os.getenv("DB_SSL_MODE") if os.getenv("DB_SSL_MODE") else clickhouse_config["sslMode"]

    # 创建连接
    clickhouse_connect = {"ip": clickhouse_ip, "port": clickhouse_port, "user": clickhouse_user}
    if clickhouse_pwd:
        clickhouse_connect["password"] = clickhouse_pwd

    graphClient = get_client(clickhouse_connect)
    graph = CHGraph(graphClient)

    if db_pwd:
        url = 'postgresql://' + db_user + ':' + db_pwd + '@' + db_ip + ':' + db_port + '/' + db_table + '?sslmode=require'
    else:
        url = 'postgresql://' + db_user + '@' + db_ip + ':' + db_port + '/' + db_table + '?sslmode=disable'

    db = DBoperator(url)

    print("服务所有配置加载结束")

    config_params = {
        "graph": graph,
        "db": db,
        "graphClient": graphClient,
        "clickhouse_connect": clickhouse_connect
    }
    return config_params


if __name__ == "__main__":
    sql = "show create table sub_graph_test_2.default_metagroups"
    configs = load_config()
    client = configs.get("graphClient")
    rest = client.execute(sql)
    print(rest)

    rest1 = client.execute("show create table default.default_metagroups")
    print("parent graph schema: {}".format(rest1))



import os
import time
import gzip
import numpy as np


def success_request(data, response):
    res = {}
    res["data"] = data if data else {}
    res["context"] = response if response else {"code": 200, "timestamp": time.time()}
    return res


def fail_request(data, response, e):
    res = {}
    res["data"] = data if data else {}

    if response:
        res["context"] = response
    else:
        message = str(e) if e else "系统异常"
        res["context"] = {"code": 500, "timestamp": time.time(), "massage": message}

    return res


def tostr(obj):
    if type(obj) == list:
        return str(obj)
    elif type(obj) == np.ndarray:
        return np.array2string(obj, separator=',', threshold=1000000000,
                               max_line_width=1000000000)


def unzip_file(file_path, file_name):
    new_file_name = file_name.replace(".gz", "")
    gzip_file = gzip.GzipFile(os.path.join(file_path, file_name))
    with open(os.path.join(file_path, new_file_name), "wb+") as f:
        f.write(gzip_file.read())

    return os.path.join(file_path, new_file_name)





import requests
import json

if __name__ == '__main__':

# graph deletion
    url_graph_deletion = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/graph-deletion"
    data_graph_deletion = json.dumps({
        "graph_name": "taobao_v1"
    })
    response = requests.delete(url=url_graph_deletion, data=data_graph_deletion)
    print(response.text)


# match edge
    url_edges = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/edges"
    data_edges = json.dumps({
        "graph_name": "taobao_v1",
        "edge_name": "user_adgroup",
        "edge_con_list": [
            "record_date='2017-05-12'",
            "record_time>'2017-05-12 23:30:00'",
            "pid='430548_1007'"],
        "target_field_list": ["record_time"]
    })
    response = requests.get(url=url_edges, data=data_edges)
    print(response.text)


# match vertices
    url_vertices = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/vertices"
    data_vertices = json.dumps({
        "graph_name": "taobao_v1",
        "vertex_name": "user",
        "vertex_con_list": [
            "final_gender_code='2'",
            "age_level='6'",
            "pvalue_level='2'",
            "shopping_level='3'",
            "occupation='0'"
        ],
        "target_field_list": [
            "new_user_class_level",
            "record_date"
        ]
    })
    response = requests.get(url=url_vertices, data=data_vertices)
    print(response.text)


# multi-hop
    url_multi_hop = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/multi-hop"
    data_multi_hop = json.dumps({
        "graph_name": "taobao_v1",
        "step": 2,
        "start_vertex_list": ["255837", "255813"],
        "edge_name_list": [
            "user_adgroup",
            "adgroup_customer"
        ],
        "direction": "backward",
        "edge_con_list_list": [
            [],
            []
        ],
        "target_field_list": [
            "record_time"
        ],
        "only_last_step": False,
        "plus_last_vertexes": False
    })
    response = requests.get(url=url_multi_hop, data=data_multi_hop)
    print(response.text)


# one-hop
    url_one_hop = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/one-hop"
    data_one_hop = json.dumps({
        "start_vertex_list": [
            "1000007",
            "100000",
            "843732"
        ],
        "edge_name_list": [
            "userid_adgroup",
            "adgroup_customer"
        ],
        "graph_name": "taobao_v1",
        "direction": "forward",
        "edge_con_list_list": [
            [
                "record_date='2017-5-13'"
            ],
            [
                "record_date='2017-5-9'"
            ]
        ],
        "target_field_list": [
            "record_time"
        ]
    })
    response = requests.get(url=url_one_hop, data=data_one_hop)
    print(response.text)


# match multi_hop_common_vertices
    url_multi_hop_common_vertices = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/multi-hop-common-vertices"
    data_multi_hop_common_vertices = json.dumps({
        "step": 1,
        "start_vertex_list": [
            "162433",
            "164028",
            "165628"
        ],
        "edge_name_list": [
            "userid_adgroup",
            "adgroup_customer"
        ],
        "graph_name": "taobao_v1",
        "direction": "forward",
        "edge_con_list_list": [[], []]
    })
    response = requests.get(url=url_multi_hop_common_vertices, data=data_multi_hop_common_vertices)
    print(response.text)


# path
    url_path = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/path"
    data_path= json.dumps({
        "graph_name": "taobao_v1",
        "start_vertex_list": ["596729","326033","596729"],
        "end_vertex_list": ["255837","255813"],
        "edge_name_list": ["user_adgroup","adgroup_customer"],
        "edge_con_list_list": [[],[]],
        "target_field_list": ["record_time"],
        "step_limit":2
    })
    response = requests.get(url=url_path, data=data_path)
    print(response.text)


# graph registration
    url_graph_registration = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/graph-registration"
    data_graph_registration = json.dumps({
        "graph_name": "taobao_v1",
        "graph_cfg": {"edges":{"adgroup_customer":{"db":"taobao","table":"adgroup_customerid_edge","src":"adgroup_id","dst":"customer_id","rank":"record_time","fields":["record_date","record_time"]      },"user_adgroup":{"db":"taobao","table":"userid_adgroup_edge","src":"user_id","dst":"adgroup_id","rank":"record_time","fields":["record_date","record_time","pid"]      }},"vertexes":{"user":{"db":"taobao","table":"user_profile","id":"user_id","fields":["record_date", "cms_segid", "cms_group_id", "final_gender_code", "age_level", "pvalue_level", "shopping_level", "occupation", "new_user_class_level"]},"adgroup":{"db":"taobao","table":"ad_feature","id":"adgroup_id","fields":["record_date","cate_id","campaign_id","brand","price"]}}}
    })
    response = requests.post(url=url_graph_registration, data=data_graph_registration)
    print(response.text)


# graph show
    url_graph_show = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/graph-show"
    data_graph_show = json.dumps({})
    response = requests.get(url=url_graph_show, data=data_graph_show)
    print(response.text)


# graph summary
    url_graph_summary = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/graph-summary/taobao_v1"
    # data_graph_summary = json.dumps({
    #     "graph_name": "livejournal_v1"
    # })
    response = requests.get(url=url_graph_summary)
    print(response.text)


# graph description
    url_graph_description = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/graph-description/taobao_v1"
    # data_graph_description = json.dumps({
    #     "graph_name": "livejournal_v1"
    # })
    response = requests.get(url=url_graph_description)
    print(response.text)


# subgraph creation
    url_subgraph_creation = "p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/subgraph-creation"
    data_subgraph_creation = json.dumps({
        "graph_name": "taobao_v1",
        "subgraph_name":"taobao_v1_sub"
    })
    response = requests.get(url=data_subgraph_creation)
    print(response.text)

# subgraph destruction
    url_subgraph_destruction = "p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/subgraph-destruction"
    data_subgraph_destruction = json.dumps({
        "graph_name": "taobao_v1",
        "subgraph_name":"taobao_v1_sub"
    })
    response = requests.get(url=data_subgraph_destruction)
    print(response.text)

# subgraph update multi hop
    url_subgraph_update_multi_hop = "p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/subgraph-update-multi-hop"
    data_subgraph_update_multi_hop = json.dumps({
            "graph_name": "taobao_v1",
            "subgraph_name":"taobao_v1_sub",
            "step": 2,
            "start_vertex_list": ["255837", "255813"],
            "edge_name_list": [
                "user_adgroup",
                "adgroup_customer"
            ],
            "direction": "backward",
            "edge_con_list_list": [
                [],
                []
            ]
    })
    response = requests.get(url=data_subgraph_update_multi_hop)
    print(response.text)

# subgraph update path
    url_subgraph_update_path = "p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/subgraph-update-path"
    data_subgraph_update_path = json.dumps({
        "graph_name": "taobao_v1",
        "subgraph_name":"taobao_v1_sub",
        "start_vertex_list": ["596729", "326033", "596729"],
        "end_vertex_list": ["255837", "255813"],
        "edge_name_list": ["user_adgroup", "adgroup_customer"],
        "edge_con_list_list": [[], []],
        "step_limit": 2
    })
    response = requests.get(url=data_subgraph_update_path)
    print(response.text)


# subgraph update edge
    url_subgraph_update_edge = "p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/subgraph-update-edge"
    data_subgraph_update_edge = json.dumps({
        "graph_name": "taobao_v1",
        "subgraph_name": "taobao_v1_sub",
        "edge_name": "user_adgroup",
        "edge_con_list": [
            "record_date='2017-05-12'",
            "record_time>'2017-05-12 23:30:00'",
            "pid='430548_1007'"
        ]
    })
    response = requests.get(url=data_subgraph_update_edge)
    print(response.text)



import requests
import json

if __name__ == '__main__':


# graph deletion
    url_graph_deletion = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/graph-deletion"
    data_graph_deletion = json.dumps({
        "graph_name": "cyber3"
    })
    response = requests.delete(url=url_graph_deletion, data=data_graph_deletion)
    print(response.text)


# match edge
    url_edges = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/edges"
    data_edges = json.dumps({
        "edge_name": "tcpflow",
        "graph_name": "cyber3",
        "edge_con_list": [
            "downlink_length>100000000"
        ],
        "target_field_list": [
            "record_time",
            "downlink_length"
        ]
    })
    response = requests.get(url=url_edges, data=data_edges)
    print(response.text)


# match vertices
#     url_edges = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/vertices"
#     data_edges = json.dumps(
#       {
#
#       }
#     )
#     response = requests.get(url=url_edges, data=data_edges)
#     print(response.text)


# multi-hop
    url_multi_hop = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/multi-hop"
    data_multi_hop = json.dumps({
        "step": 2,
        "start_vertex_list": [
            "10.73.28.115",
            "10.78.55.20"
        ],
        "edge_name_list": [
            "tcpflow",
            "flow"
        ],
        "graph_name": "cyber3",
        "direction": "forward",
        "edge_con_list_list": [
            [
                "downlink_length>10000",
                "protocol='http'"
            ],
            [
                "record_date='2019-04-15'"
            ]
        ],
        "target_field_list": [
            "record_time"
        ],
        "only_last_step": True,
        "plus_last_vertexes": True
    })
    response = requests.get(url=url_multi_hop, data=data_multi_hop)
    print(response.text)


# match one-hop
    url_one_hop = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/one-hop"
    data_one_hop = json.dumps({
        "start_vertex_list": [
            "10.73.28.115",
            "10.78.55.20"
        ],
        "edge_name_list": [
            "tcpflow",
            "flow"
        ],
        "graph_name": "cyber3",
        "direction": "forward",
        "edge_con_list_list": [
            [
                "downlink_length>1000",
                "protocol='http'"
            ],
            [
                "record_date='2019-04-15'"
            ]
        ],
        "target_field_list": [
            "record_time"
        ]
    })
    response = requests.get(url=url_one_hop, data=data_one_hop)
    print(response.text)


# match multi_hop_common_vertices
    url_multi_hop_common_vertices = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/multi-hop-common-vertices"
    data_multi_hop_common_vertices = json.dumps({
        "step": 2,
        "start_vertex_list": [
            "10.73.28.115",
            "10.78.55.20"
        ],
        "edge_name_list": [
            "tcpflow",
            "flow"
        ],
        "graph_name": "cyber3",
        "direction": "forward",
        "edge_con_list_list": [
            [
                "protocol='http'"
            ],
            [
                "record_date='2019-04-15'"
            ]
        ],
        "only_last_step":False
    })
    response = requests.get(url=url_multi_hop_common_vertices, data=data_multi_hop_common_vertices)
    print(response.text)


# path
    url_path = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/path"
    data_path= json.dumps({
        "step_limit": 2,
        "start_vertex_list": [
            "10.73.28.115"
        ],
        "end_vertex_list": [
            "106.38.199.41",
            "10.59.81.218"
  ],
        "edge_name_list": [
            "tcpflow",
            "flow"
        ],
        "graph_name": "cyber3",
        "edge_con_list_list": [
            [
                "downlink_length>10000",
                "protocol='http'"
            ],
            [
                "record_date='2019-04-15'"
            ]
        ],
        "target_field_list": [
            "record_time"
        ]
    })
    response = requests.get(url=url_path, data=data_path)
    print(response.text)


# graph registration
    url_graph_registration = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/graph-registration"
    data_graph_registration = json.dumps({
        "graph_name": "cyber_10",
        "graph_cfg": "{\"edges\":{\"tcpflow\":{\"db\":\"graph\",\"table\":\"tcpflow\",\"src\":\"source_ip\",\"dst\":\"destination_ip\",\"rank\":\"record_time\",\"fields\":[\"record_date\",\"record_time\",\"protocol\",\"destination_port\",\"uplink_length\",\"downlink_length\"] },\"flow\":{\"db\":\"graph\",\"table\":\"flow\",\"src\":\"source_ip\",\"dst\":\"destination_ip\", \"rank\":\"record_time\",\"fields\":[\"record_date\",\"record_time\",\"destination_port\",\"uri\"]}},\"vertexes\":{}}"
    })
    response = requests.post(url=url_graph_registration, data=data_graph_registration)
    print(response.text)


# graph show
    url_graph_show = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/graph-show"
    data_graph_show = json.dumps({})
    response = requests.get(url=url_graph_show, data=data_graph_show)
    print(response.text)


# graph summary
    url_graph_summary = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/graph-summary/cyber3"
    # data_graph_summary = json.dumps({
    #     "graph_name": "livejournal_v1"
    # })
    response = requests.get(url=url_graph_summary)
    print(response.text)


# graph description
    url_graph_description = "http://p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/graph-description/cyber3"
    # data_graph_description = json.dumps({
    #     "graph_name": "livejournal_v1"
    # })
    response = requests.get(url=url_graph_description)
    print(response.text)


# subgraph creation
    url_subgraph_creation = "p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/subgraph-creation"
    data_subgraph_creation = json.dumps({
        "graph_name": "cyber_10",
        "subgraph_name":"cyber_10_sub1"
    })
    response = requests.get(url=data_subgraph_creation)
    print(response.text)

# subgraph destruction
    url_subgraph_destruction = "p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/subgraph-destruction"
    data_subgraph_destruction = json.dumps({
        "graph_name": "cyber_10",
        "subgraph_name":"cyber_10_sub1"
    })
    response = requests.get(url=data_subgraph_destruction)
    print(response.text)

# subgraph update multi hop
    url_subgraph_update_multi_hop = "p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/subgraph-update-multi-hop"
    data_subgraph_update_multi_hop = json.dumps({
        "step": 2,
        "start_vertex_list": [
            "10.73.28.115",
            "10.78.55.20"
        ],
        "edge_name_list": [
            "tcpflow",
            "flow"
        ],
        "graph_name": "cyber_10",
        "subgraph_name": "cyber_10_sub1",
        "direction": "forward",
        "edge_con_list_list": [
            [
                "downlink_length>10000",
                "protocol='http'"
            ],
            [
                "record_date='2019-04-15'"
            ]
        ]
    })
    response = requests.get(url=data_subgraph_update_multi_hop)
    print(response.text)

# subgraph update path
    url_subgraph_update_path = "p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/subgraph-update-path"
    data_subgraph_update_path = json.dumps({
            "step_limit": 2,
            "start_vertex_list": [
                "10.73.28.115"
            ],
            "end_vertex_list": [
                "106.38.199.41",
                "10.59.81.218"
            ],
            "edge_name_list": [
                "tcpflow",
                "flow"
            ],
            "graph_name": "cyber_10",
            "subgraph_name": "cyber_10_sub1",
            "edge_con_list_list": [
                [
                    "downlink_length>10000",
                    "protocol='http'"
                ],
                [
                    "record_date='2019-04-15'"
                ]
            ]
    })
    response = requests.get(url=data_subgraph_update_path)
    print(response.text)


# subgraph update edge
    url_subgraph_update_edge = "p47708v.hulk.shbt.qihoo.net:10010/graph-db/api/v1/subgraph-update-edge"
    data_subgraph_update_edge = json.dumps({
        "edge_name": "tcpflow",
        "graph_name": "cyber_10",
        "subgraph_name": "cyber_10_sub1",
        "edge_con_list": [
            "downlink_length>100000000"
        ]
    })
    response = requests.get(url=data_subgraph_update_edge)
    print(response.text)





def deal_query_result(data_type, client, sql):
    if data_type == "list":
        res = client.execute(sql)
    elif data_type == "df":
        res = {}
        result = client.query_dataframe(sql)
        res['schema'] = result.columns.values.tolist()
        res['detail'] = result.values.tolist()
    else:
        res = None

    return res


def deal_query_result_sample(data_type, client, sql):
    if data_type == "list":
        res = client.execute(sql)
    elif data_type == "df":
        res = client.query_dataframe(sql)
    else:
        res = None

    return res


def deal_query_result_sample_plus(data_type, client, sql):
    if data_type == "list":
        res = client.execute(sql)
    elif data_type == "df":
        res = client.query_dataframe(sql).values.tolist()
    else:
        res = None

    return res


def deal_query_result_with_paging(data_type, client, sql, page, page_size, count):
    if data_type == "list":
        res = client.execute(sql)
    elif data_type == "df":
        res = {}
        result = client.query_dataframe(sql)
        res['schema'] = result.columns.values.tolist()
        res['detail'] = result.values.tolist()
        res['page'] = page
        res['page_size'] = page_size
        res['count'] = count
    else:
        res = None

    return res



from contextlib import contextmanager
from clickhouse_driver import Client


def get_client(clickhouse_connect):
    if "user" in clickhouse_connect and "password" in clickhouse_connect:
        graph_client = Client(host=clickhouse_connect["ip"], port=clickhouse_connect["port"], user=clickhouse_connect["user"],
                              password=clickhouse_connect["password"])
    else:
        graph_client = Client(host=clickhouse_connect["ip"], port=clickhouse_connect["port"])
    graph_client.execute(" set max_query_size=100000000 ")
    return graph_client


@contextmanager
def get_client_new(clickhouse_connect):
    graph_client = None
    try:
        if "user" in clickhouse_connect and "password" in clickhouse_connect:
            graph_client = Client(host=clickhouse_connect.get("ip"), port=clickhouse_connect.get("port"),
                                  user=clickhouse_connect.get("user"),
                                  password=clickhouse_connect.get("password"))
        else:
            graph_client = Client(host=clickhouse_connect["ip"], port=clickhouse_connect["port"])
        graph_client.execute(" set max_query_size=100000000 ")
        yield graph_client
    except Exception as e:
        print("get clickhouse client error: {}".format(str(e)))
    finally:
        if graph_client:
            graph_client.disconnect()


import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from celery_tasks.celery_app import app
from celery_tasks.plato_client import PlatoClient


@app.task
def graph_calculation_task(sub_graph, task_id, calculation_type):
    plato_client = PlatoClient(sub_graph, task_id, calculation_type)
    plato_client.run()


if __name__ == "__main__":
    graph_calculation_task.delay("7cf53539_81d6_4986_9745_86d2acfc97c6", 8, 'pagerank')



